"""
The default/recommended SocketServer-based server implementation. 
"""
import logging
import socket
import threading
from SocketServer import BaseRequestHandler, TCPServer, ThreadingMixIn

from stompclient.util import FrameBuffer

from coilmq.server import StompConnection
from coilmq.engine import StompEngine

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class StompRequestHandler(BaseRequestHandler, StompConnection):
    """
    Class that will be instantiated to handle STOMP connections.

    This class will be instantiated once per connection to the server.  In a multi-threaded
    context, that means that instances of this class are scoped to a single thread.  It should
    be noted that while the L{coilmq.engine.StompEngine} instance will be thread-local, the 
    storage containers configured into the engine are not thread-local (and hence must be
    thread-safe). 
    
    @ivar buffer: A StompBuffer instance which buffers received data (to ensure we deal with
                    complete STOMP messages.
    @type buffer: C{stompclient.util.FrameBuffer}
    
    @ivar engine: The STOMP protocol engine.
    @type engine: L{coilmq.engine.StompEngine}
    
    @ivar debug: Whether to enable extra-verbose debug logging.  (Will be logged at debug level.)
    @type debug: C{bool}
    """
    
    def setup(self):
        if self.server.timeout is not None:
            self.request.settimeout(self.server.timeout)
        self.debug = False
        self.log = logging.getLogger('%s.%s' % (self.__module__, self.__class__.__name__))
        self.buffer = FrameBuffer()
        self.engine = StompEngine(connection=self,
                                  authenticator=self.server.authenticator,
                                  queue_manager=self.server.queue_manager,
                                  topic_manager=self.server.topic_manager)
        
    def handle(self):
        """
        Handle a new socket connection.
        """
        # self.request is the TCP socket connected to the client
        try:
            while not self.server._shutdown_request_event.is_set():
                try:
                    data = self.request.recv(8192)
                    if not data:
                        break
                    if self.debug:
                        self.log.debug("RECV: %r" % data)
                    self.buffer.append(data)
                    
                    for frame in self.buffer:
                        self.log.debug("Processing frame: %s" % frame)
                        self.engine.process_frame(frame)
                except socket.timeout:
                    pass
        except Exception, e:
            self.log.error("Error receiving data (unbinding): %s" % e)
            self.engine.unbind()
            raise
    
    def finish(self):
        """
        Normal (non-error) termination of request.
        
        Unbinds the engine.
        @see: L{coilmq.engine.StompEngine.unbind}
        """
        self.engine.unbind()
        
    def send_frame(self, frame):
        """ Sends a frame to connected socket client.
        
        @param frame: The frame to send.
        @type frame: C{stompclient.frame.Frame}
        """
        packed = frame.pack()
        if self.debug:
            self.log.debug("SEND: %r" % packed)
        self.request.sendall(packed)

class StompServer(TCPServer):
    """
    Subclass of C{StompServer.TCPServer} to handle new connections with 
    instances of L{StompRequestHandler}.
    
    @ivar authenticator: The authenticator to use.
    @type authenticator: L{coilmq.auth.Authenticator}
    
    @ivar queue_manager: The queue manager to use.
    @type queue_manager: L{coilmq.queue.QueueManager}
    
    @ivar topic_manager: The topic manager to use.
    @type topic_manager: L{coilmq.topic.TopicManager}
    """
    
    # This causes the SO_REUSEADDR option to be set on the socket, allowing
    # server to rebind to the same address (w/o waiting for connections to 
    # leave TIME_WAIT after unclean disconnect).
    allow_reuse_address = True
    
    def __init__(self, server_address, RequestHandlerClass=None, timeout=3.0, authenticator=None, queue_manager=None, topic_manager=None):
        """
        Extension to C{TCPServer} constructor to provide mechanism for providing implementation classes.
        
        @param server_address: The (address,port) C{tuple}
        @param RequestHandlerClass: The class to use for handling requests.
        @param timeout: The timeout for the underlying socket.
        @keyword authenticator: The configure L{coilmq.auth.Authenticator} object to use.
        @keyword queue_manager: The configured L{coilmq.queue.QueueManager} object to use.
        @keyword topic_manager: The configured L{coilmq.topic.TopicManager} object to use. 
        """
        self.log = logging.getLogger('%s.%s' % (self.__module__, self.__class__.__name__))
        if not RequestHandlerClass:
            RequestHandlerClass = StompRequestHandler
        TCPServer.__init__(self, server_address, RequestHandlerClass)
        self.timeout = timeout
        self.authenticator = authenticator
        self.queue_manager = queue_manager
        self.topic_manager = topic_manager
        self._shutdown_request_event = threading.Event()
    
    def server_close(self):
        """
        Closes the socket server and any associated resources.
        """
        self.log.debug("Closing the socket server connection.")
        TCPServer.server_close(self)
        self.queue_manager.close()
        self.topic_manager.close()
        if hasattr(self.authenticator, 'close'):
            self.authenticator.close()
        self.shutdown()
    
    def shutdown(self):
        self._shutdown_request_event.set()
        TCPServer.shutdown(self)
        
    def serve_forever(self, poll_interval=0.5):
        """Handle one request at a time until shutdown.

        Polls for shutdown every poll_interval seconds. Ignores
        self.timeout. If you need to do periodic tasks, do them in
        another thread.
        """
        self._shutdown_request_event.clear()
        TCPServer.serve_forever(self, poll_interval=poll_interval)
        
class ThreadedStompServer(ThreadingMixIn, StompServer):
    pass


"""
Storage containers for durable queues and (planned) durable topics.
"""
import abc
import logging
import threading

from coilmq.util.concurrency import synchronized

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class QueueStore(object):
    """
    Abstract base class for queue storage. 
    
    Extensions/implementations of this class must be thread-safe.
    
    @ivar log: A logger for this class.
    @type log: C{logging.Logger}
    """
    __metaclass__ = abc.ABCMeta
    
    def __init__(self):
        """
        A base constructor that sets up logging and the lock used by synchronized decorator.
        
        If you extend this class, you should either call this method or at minimum make sure these values
        get set.
        """
        self.log = logging.getLogger('%s.%s' % (self.__module__, self.__class__.__name__))
        self._lock = threading.RLock()
    
    @abc.abstractmethod
    @synchronized
    def enqueue(self, destination, frame):
        """
        Store message (frame) for specified destinationination.
        
        @param destination: The destinationination queue name for this message (frame).
        @type destination: C{str}
        
        @param frame: The message (frame) to send to specified destinationination.
        @type frame: C{stompclient.frame.Frame}
        """
    
    @abc.abstractmethod
    @synchronized
    def dequeue(self, destination):
        """
        Removes and returns an item from the queue (or C{None} if no items in queue).
        
        @param destination: The queue name (destinationination).
        @type destination: C{str}
        
        @return: The first frame in the specified queue, or C{None} if there are none.
        @rtype: C{stompclient.frame.Frame} 
        """
    
    @synchronized
    def requeue(self, destination, frame):
        """
        Requeue a message (frame) for storing at specified destinationination.
        
        @param destination: The destinationination queue name for this message (frame).
        @type destination: C{str}
        
        @param frame: The message (frame) to send to specified destinationination.
        @type frame: C{stompclient.frame.Frame}
        """
        self.enqueue(destination, frame)

    @synchronized
    def size(self, destination):
        """
        Size of the queue for specified destination.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        
        @return: The number of frames in specified queue.
        @rtype: C{int}
        """
        raise NotImplementedError()

    @synchronized
    def has_frames(self, destination):
        """
        Whether specified destination has any frames.
        
        Default implementation uses L{QueueStore.size} to determine if there
        are any frames in queue.  Subclasses may choose to optimize this.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        
        @return: The number of frames in specified queue.
        @rtype: C{int}
        """
        return self.size(destination) > 0
    
    @synchronized
    def close(self):
        """
        May be implemented to perform any necessary cleanup operations when store is closed.
        """
        pass
    
    # This is intentionally not synchronized, since it does not directly
    # expose any shared data.
    def frames(self, destination):
        """
        Returns an iterator for frames in specified queue.
        
        The iterator simply wraps calls to L{dequeue} method, so the order of the 
        frames from the iterator will be the reverse of the order in which the
        frames were enqueued.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        """
        return QueueFrameIterator(self, destination)
    
class QueueFrameIterator(object):
    """
    Provides an C{iterable} over the frames for a specified destination in a queue.
    
    @ivar store: The queue store.
    @type store: L{coilmq.store.QueueStore}
    
    @ivar destination: The destination for this iterator.
    @type destination: C{str}
    """
    def __init__(self, store, destination):
        self.store = store
        self.destination = destination
        
    def __iter__(self):
        return self
    
    def next(self):
        frame = self.store.dequeue(self.destination)
        if not frame:
            raise StopIteration()
        return frame
    
    def __len__(self):
        return self.store.size(self.destination)
    
    def __nonzero__(self):
        return self.store.has_frames(self.destination)
    
class TopicStore(object):
    """
    Abstract base class for non-durable topic storage.
    """
    
class DurableTopicStore(TopicStore):
    """
    Abstract base class for durable topic storage.
    """


"""
Test SQLAlchemy storage.
"""
import unittest
import tempfile
import shutil
import uuid
import time
import datetime

from sqlalchemy import create_engine
from stompclient.frame import Frame

from coilmq.store.sa import meta, model
from coilmq.store.sa import init_model
from coilmq.store.sa import SAQueue

from coilmq.tests.store import CommonQueueTestsMixin

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class SAQueueTest(unittest.TestCase, CommonQueueTestsMixin):
    
    def setUp(self):
        engine = create_engine('sqlite:///:memory:', echo=True)
        init_model(engine)
        self.store = SAQueue()
    
    def tearDown(self):
        print dir(meta.engine)
        meta.Session.remove()
        
    def test_dequeue_order(self):
        """ Test the order that frames are returned by dequeue() method. """
        dest = '/queue/foo'
        
        frame1 = Frame('MESSAGE', headers={'message-id': 'id-1'}, body='message-1') 
        self.store.enqueue(dest, frame1)
        
        frame2 = Frame('MESSAGE', headers={'message-id': 'id-2'}, body='message-2') 
        self.store.enqueue(dest, frame2)
        
        frame3 = Frame('MESSAGE', headers={'message-id': 'id-3'}, body='message-3') 
        self.store.enqueue(dest, frame3)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 3
        
        # Perform some updates to change the expected order.
        
        sess = meta.Session()
        sess.execute(model.frames_table.update().where(model.frames_table.c.message_id=='id-1').values(queued=datetime.datetime(2010, 01, 01)))
        sess.execute(model.frames_table.update().where(model.frames_table.c.message_id=='id-2').values(queued=datetime.datetime(2009, 01, 01)))
        sess.execute(model.frames_table.update().where(model.frames_table.c.message_id=='id-3').values(queued=datetime.datetime(2004, 01, 01)))
        sess.commit()
        
        rframe1 = self.store.dequeue(dest)
        assert frame3 == rframe1
         
        rframe2 = self.store.dequeue(dest)
        assert frame2 == rframe2
        
        rframe3 = self.store.dequeue(dest)
        assert frame1 == rframe3
        
        assert self.store.has_frames(dest) == False
        assert self.store.size(dest) == 0

    


"""
Queue storage tests.
"""
import uuid

from stompclient.frame import Frame

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class CommonQueueTestsMixin(object):
    """
    An abstract set of base tests for queue storage engines.
    
    This class must be mixed in with something that extends C{unittest.TestCase}.
    """
    
    def test_enqueue(self):
        """ Test the enqueue() method. """
        dest = '/queue/foo'
        frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data')
        self.store.enqueue(dest, frame)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 1
        
    def test_dequeue(self):
        """ Test the dequeue() method. """
        dest = '/queue/foo'
        frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data') 
        self.store.enqueue(dest, frame)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 1
        
        rframe = self.store.dequeue(dest)
        assert frame == rframe
        
        # We cannot generically assert whether or not frame and rframe are
        # the *same* object. 
        
        assert self.store.has_frames(dest) == False
        assert self.store.size(dest) == 0
    
    def test_dequeue_specific(self):
        """ Test that we only dequeue from the correct queue. """
        dest = '/queue/foo'
        notdest = '/queue/other'
        
        frame1 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-1') 
        self.store.enqueue(dest, frame1)
        
        frame2 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-2') 
        self.store.enqueue(notdest, frame2)
        
        frame3 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-3') 
        self.store.enqueue(dest, frame3)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 2
        
        rframe1 = self.store.dequeue(dest)
        assert frame1 == rframe1
         
        rframe2 = self.store.dequeue(dest)
        assert frame3 == rframe2
        
        assert self.store.has_frames(dest) == False
        assert self.store.size(dest) == 0
        
    def test_dequeue_order(self):
        """ Test the order that frames are returned by dequeue() method. """
        dest = '/queue/foo'
        
        frame1 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-1') 
        self.store.enqueue(dest, frame1)
        
        frame2 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-2') 
        self.store.enqueue(dest, frame2)
        
        frame3 = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='message-3') 
        self.store.enqueue(dest, frame3)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 3
        
        rframe1 = self.store.dequeue(dest)
        assert frame1 == rframe1
         
        rframe2 = self.store.dequeue(dest)
        assert frame2 == rframe2
        
        rframe3 = self.store.dequeue(dest)
        assert frame3 == rframe3
        
        assert self.store.has_frames(dest) == False
        assert self.store.size(dest) == 0
        
    def test_dequeue_empty(self):
        """ Test dequeue() with empty queue. """
        
        r = self.store.dequeue('/queue/nonexist')
        assert r is None
        
        assert self.store.has_frames('/queue/nonexist') == False
        assert self.store.size('/queue/nonexist') == 0


"""
Tests for authenticators.
"""
import unittest
from StringIO import StringIO

from pkg_resources import resource_stream, resource_filename

from coilmq.auth.simple import SimpleAuthenticator

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class SimpleAuthenticatorTest(unittest.TestCase):
    
    def setUp(self):
        pass
        
    def tearDown(self):
        pass
    
    def test_constructor(self):
        """
        Test the with passing auth store in constructor.
        """
        auth = SimpleAuthenticator({'user': 'pass'})
        assert auth.authenticate('user', 'pass') == True
        assert auth.authenticate('user1', 'pass') == False
        assert auth.authenticate('user', 'pass2') == False

    def test_from_configfile(self):
        """
        Test the loading store from config file path.
        """
        filename = resource_filename('coilmq.tests.resources', 'auth.ini')
        auth = SimpleAuthenticator()
        auth.from_configfile(filename)
        print "Auth store: %r" % auth.store
        assert auth.authenticate('juniper', 'b3rr1es') == True
        assert auth.authenticate('oak', 'ac$rrubrum') == True
        assert auth.authenticate('pinetree', 'str0bus') == True
        assert auth.authenticate('foo', 'bar') == False
        
    def test_from_configfile_fp(self):
        """
        Test loading store from file-like object. 
        """
        fp = resource_stream('coilmq.tests.resources', 'auth.ini')
        auth = SimpleAuthenticator()
        auth.from_configfile(fp)
        
        assert auth.authenticate('juniper', 'b3rr1es') == True
        assert auth.authenticate('oak', 'ac$rrubrum') == True
        assert auth.authenticate('pinetree', 'str0bus') == True
        assert auth.authenticate('foo', 'bar') == False
    
    def test_from_configfile_invalid(self):
        """
        Test loading store with invalid file path.
        """
        filename = resource_filename('coilmq.tests.resources', 'auth-invlaid.ini')
        auth = SimpleAuthenticator()
        try:
            auth.from_configfile(filename)
            self.fail("Expected error with invalid filename.")
        except ValueError, e:
            pass
        
    def test_from_configfile_fp_invalid(self):
        """
        Test loading store with missing section in config.
        """
        buf = "[invalid]\nusername=password"
        fp = StringIO(buf)
        auth = SimpleAuthenticator()
        try:
            auth.from_configfile(fp)
            self.fail("Expected error with missing section.")
        except ValueError, e:
            pass


"""
Non-durable topic support functionality.

This code is inspired by the design of the Ruby stompserver project, by 
Patrick Hurley and Lionel Bouton.  See http://stompserver.rubyforge.org/
"""
import logging
import threading
import uuid
from collections import defaultdict

from coilmq.util.concurrency import synchronized

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class TopicManager(object):
    """
    Class that manages distribution of messages to topic subscribers.
    
    This class uses C{threading.RLock} to guard the public methods.  This is probably
    a bit excessive, given 1) the actomic nature of basic C{dict} read/write operations 
    and  2) the fact that most of the internal data structures are keying off of the 
    STOMP connection, which is going to be thread-isolated.  That said, this seems like 
    the technically correct approach and should increase the chance of this code being
    portable to non-GIL systems.
    
    @ivar _topics: A dict of registered topics, keyed by destination.
    @type _topics: C{dict} of C{str} to C{set} of L{coilmq.server.StompConnection}
    """
    def __init__(self):
        self.log = logging.getLogger('%s.%s' % (__name__, self.__class__.__name__))
        
        # Lock var is required for L{synchornized} decorator. 
        self._lock = threading.RLock()
        
        self._topics = defaultdict(set)
    
        # TODO: If we want durable topics, we'll need a store for topics.
    
    @synchronized
    def close(self):
        """
        Closes all resources associated with this topic manager.
        
        (Currently this is simply here for API conformity w/ L{coilmq.queue.QueueManager}.)
        """
        pass
    
    @synchronized
    def subscribe(self, connection, destination):
        """
        Subscribes a connection to the specified topic destination. 
        
        @param connection: The client connection to subscribe.
        @type connection: L{coilmq.server.StompConnection}
        
        @param destination: The topic destination (e.g. '/topic/foo')
        @type destination: C{str} 
        """
        self.log.debug("Subscribing %s to %s" % (connection, destination))
        self._topics[destination].add(connection)
    
    @synchronized
    def unsubscribe(self, connection, destination):
        """
        Unsubscribes a connection from the specified topic destination. 
        
        @param connection: The client connection to unsubscribe.
        @type connection: L{coilmq.server.StompConnection}
        
        @param destination: The topic destination (e.g. '/topic/foo')
        @type destination: C{str} 
        """
        self.log.debug("Unsubscribing %s from %s" % (connection, destination))
        if connection in self._topics[destination]:
            self._topics[destination].remove(connection)
        
        if not self._topics[destination]:
            del self._topics[destination]
    
    @synchronized        
    def disconnect(self, connection):
        """
        Removes a subscriber connection.
        
        @param connection: The client connection to unsubscribe.
        @type connection: L{coilmq.server.StompConnection}
        """
        self.log.debug("Disconnecting %s" % connection)
        for dest in self._topics.keys():
            if connection in self._topics[dest]:
                self._topics[dest].remove(connection)
            if not self._topics[dest]: 
                del self._topics[dest] # This won't trigger RuntimeError, since we're using items()

    @synchronized
    def send(self, message):
        """
        Sends a message to all subscribers of destination.
        
        @param message: The message frame.  (The frame will be modified to set command 
                            to MESSAGE and set a message id.)
        @type message: L{stompclient.frame.Frame}
        """
        dest = message.destination
        if not dest:
            raise ValueError("Cannot send frame with no destination: %s" % message)
        
        message.command = 'MESSAGE'
        
        if not 'message-id' in message.headers:
            message.headers['message-id'] = str(uuid.uuid4())
            
        for subscriber in self._topics[dest]:
            subscriber.send_frame(message)    
    


"""
Queue manager, queue implementation, and supporting classes.

This code is inspired by the design of the Ruby stompserver project, by 
Patrick Hurley and Lionel Bouton.  See http://stompserver.rubyforge.org/
"""
import logging
import threading
import uuid

from collections import defaultdict

from coilmq.scheduler import FavorReliableSubscriberScheduler, RandomQueueScheduler
from coilmq.util.concurrency import synchronized

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class QueueManager(object):
    """
    Class that manages distribution of messages to queue subscribers.
    
    This class uses C{threading.RLock} to guard the public methods.  This is probably
    a bit excessive, given 1) the actomic nature of basic C{dict} read/write operations 
    and  2) the fact that most of the internal data structures are keying off of the 
    STOMP connection, which is going to be thread-isolated.  That said, this seems like 
    the technically correct approach and should increase the chance of this code being
    portable to non-GIL systems. 
    
    @ivar store: The queue storage backend to use.
    @type store: L{coilmq.store.QueueStore}
        
    @ivar subscriber_scheduler: The scheduler that chooses which subscriber to send
                                    messages to.
    @type subscriber_scheduler: L{coilmq.scheduler.SubscriberPriorityScheduler}
        
    @ivar queue_scheduler: The scheduler that chooses which queue to select for sending
                                    backlogs for a single connection.
    @type queue_scheduler: L{coilmq.scheduler.QueuePriorityScheduler}
    
    @ivar _queues: A dict of registered queues, keyed by destination.
    @type _queues: C{dict} of C{str} to C{set} of L{coilmq.server.StompConnection}
        
    @ivar _pending: All messages waiting for ACK from clients.
    @type _pending: C{dict} of L{coilmq.server.StompConnection} to C{stompclient.frame.Frame}
    
    @ivar _transaction_frames: Frames that have been ACK'd within a transaction.
    @type _transaction_frames: C{dict} of L{coilmq.server.StompConnection} to C{dict} of C{str} to C{stompclient.frame.Frame}
    """
    
    def __init__(self, store, subscriber_scheduler=None, queue_scheduler=None):
        """
        @param store: The queue storage backend.
        @type store: L{coilmq.store.QueueStore}
        
        @param subscriber_scheduler: The scheduler that chooses which subscriber to send
                                    messages to.
        @type subscriber_scheduler: L{coilmq.scheduler.SubscriberPriorityScheduler}
        
        @param queue_scheduler: The scheduler that chooses which queue to select for sending
                                    backlogs for a single connection.
        @type queue_scheduler: L{coilmq.scheduler.QueuePriorityScheduler}
        """
        self.log = logging.getLogger('%s.%s' % (__name__, self.__class__.__name__))
        
        
        # Use default schedulers, if they're not specified
        if subscriber_scheduler is None:
            subscriber_scheduler = FavorReliableSubscriberScheduler()
        
        if queue_scheduler is None:
            queue_scheduler = RandomQueueScheduler()
            
        # This lock var is required by L{synchronized} decorator.
        self._lock = threading.RLock()
        
        self.store = store
        self.subscriber_scheduler = subscriber_scheduler
        self.queue_scheduler = queue_scheduler
        
        self._queues = defaultdict(set)
        self._transaction_frames = defaultdict(lambda: defaultdict(list))
        self._pending = {}
    
    @synchronized
    def close(self):
        """
        Closes all resources/backends associated with this queue manager.
        """
        self.log.info("Shutting down queue manager.")
        if hasattr(self.store, 'close'):
            self.store.close()
            
        if hasattr(self.subscriber_scheduler, 'close'):
            self.subscriber_scheduler.close()
            
        if hasattr(self.queue_scheduler, 'close'):
            self.queue_scheduler.close()
        
    @synchronized
    def subscribe(self, connection, destination):
        """
        Subscribes a connection to the specified destination (topic or queue). 
        
        @param connection: The connection to subscribe.
        @type connection: L{coilmq.server.StompConnection}
        
        @param destination: The topic/queue destination (e.g. '/queue/foo')
        @type destination: C{str} 
        """
        self.log.debug("Subscribing %s to %s" % (connection, destination))
        self._queues[destination].add(connection)
        self._send_backlog(connection, destination)
    
    @synchronized
    def unsubscribe(self, connection, destination):
        """
        Unsubscribes a connection from a destination (topic or queue).
        
        @param connection: The client connection to unsubscribe.
        @type connection: L{coilmq.server.StompConnection}
        
        @param destination: The topic/queue destination (e.g. '/queue/foo')
        @type destination: C{str} 
        """
        self.log.debug("Unsubscribing %s from %s" % (connection, destination))
        if connection in self._queues[destination]:
            self._queues[destination].remove(connection)
        
        if not self._queues[destination]:
            del self._queues[destination]
    
    @synchronized
    def disconnect(self, connection):
        """
        Removes a subscriber connection, ensuring that any pending commands get requeued.
        
        @param connection: The client connection to unsubscribe.
        @type connection: L{coilmq.server.StompConnection}
        """
        self.log.debug("Disconnecting %s" % connection)
        if connection in self._pending:
            self.store.requeue(self._pending[connection])
            del self._pending[connection]
        
        for dest in self._queues.keys():
            if connection in self._queues[dest]:
                self._queues[dest].remove(connection)
            if not self._queues[dest]:
                del self._queues[dest] # This won't trigger RuntimeError, since we're using keys()
    
    
    @synchronized
    def send(self, message):
        """
        Sends a MESSAGE frame to an eligible subscriber connection.
        
        Note that this method will modify the incoming message object to 
        add a message-id header (if not present) and to change the command
        to 'MESSAGE' (if it is not).
         
        @param message: The message frame.
        @type message: C{stompclient.frame.Frame}
        """
        dest = message.destination
        if not dest:
            raise ValueError("Cannot send frame with no destination: %s" % message)
        
        message.command = 'MESSAGE'
        
        if not 'message-id' in message.headers:
            message.headers['message-id'] = str(uuid.uuid4())
        
        # Grab all subscribers for this destination that do not have pending frames
        subscribers = [s for s in self._queues[dest] 
                                    if s not in self._pending]
        
        if not subscribers:
            self.log.debug("No eligible subscribers; adding message %s to queue %s" % (message, dest))
            self.store.enqueue(dest, message)
        else:
            selected = self.subscriber_scheduler.choice(subscribers, message)
            self.log.debug("Delivering message %s to subscriber %s" % (message, selected))
            self._send_frame(selected, message)
    
    @synchronized
    def ack(self, connection, frame, transaction=None):
        """
        Acknowledge receipt of a message.
        
        If the `transaction` parameter is non-null, the frame being ack'd
        will be queued so that it can be requeued if the transaction
        is rolled back. 
        
        @param connection: The connection that is acknowledging the frame.
        @type connection: L{coilmq.server.StompConnection}
        
        @param frame: The frame being acknowledged.
        
        """
        self.log.debug("ACK %s for %s" % (frame, connection))
        
        if connection in self._pending:
            pending_frame = self._pending[connection]
            # Make sure that the frame being acknowledged matches
            # the expected frame
            if pending_frame.message_id != frame.message_id:
                self.log.warning("Got a ACK for unexpected message-id: %s" % frame.message_id)
                self.store.requeue(pending_frame.destination, pending_frame)
                # (The pending frame will be removed further down)
            
            if transaction is not None:
                self._transaction_frames[connection][transaction].append(pending_frame)
            
            del self._pending[connection]            
            self._send_backlog(connection)
            
        else:
            self.log.debug("No pending messages for %s" % connection)
    
    @synchronized
    def resend_transaction_frames(self, connection, transaction):
        """
        Resend the messages that were ACK'd in specified transaction.
        
        This is called by the engine when there is an abort command.
        
        @param connection: The client connection that aborted the transaction.
        @type connection: L{coilmq.server.StompConnection}
        
        @param transaction: The transaction id (which was aborted).
        @type transaction: C{str}
        """
        for frame in self._transaction_frames[connection][transaction]:
            self.send(frame)
    
    @synchronized
    def clear_transaction_frames(self, connection, transaction):
        """
        Clears out the queued ACK frames for specified transaction. 
        
        This is called by the engine when there is a commit command.
        
        @param connection: The client connection that committed the transaction.
        @type connection: L{coilmq.server.StompConnection}
        
        @param transaction: The transaction id (which was committed).
        @type transaction: C{str}
        """
        
        del self._transaction_frames[connection][transaction]
        
    def _send_backlog(self, connection, destination=None):
        """
        Sends any queued-up messages for the (optionally) specified destination to connection.
        
        If the destination is not provided, a destination is chosen using the 
        L{QueueManager.queue_scheduler} scheduler algorithm.
        
        (This method assumes it is being called from within a lock-guarded public
        method.)  
        
        @param connection: The client connection.
        @type connection: L{coilmq.server.StompConnection}
        
        @param destination: The topic/queue destination (e.g. '/queue/foo')
        @type destination: C{str} 
        
        @raise Exception: if the underlying connection object raises an error, the message
                            will be re-queued and the error will be re-raised.  
        """
        if destination is None:
            # Find all destinations that have frames and that contain this
            # connection (subscriber).
            eligible_queues = dict([(dest,q) for (dest, q) in self._queues.items()
                                    if connection in q and self.store.has_frames(dest)])
            destination = self.queue_scheduler.choice(eligible_queues, connection)
            if destination is None:
                self.log.debug("No eligible queues (with frames) for subscriber %s" % connection)
                return
            
        self.log.debug("Sending backlog to %s for destination %s" % (connection, destination))
        if connection.reliable_subscriber:
            # only send one message (waiting for ack)
            frame = self.store.dequeue(destination)
            if frame:
                try:
                    self._send_frame(connection, frame)
                except Exception, x:
                    self.log.error("Error sending message %s (requeueing): %s" % (frame, x))
                    self.store.requeue(destination, frame)
                    raise
        else:
            for frame in self.store.frames(destination):
                try:
                    self._send_frame(connection, frame)
                except Exception, x:
                    self.log.error("Error sending message %s (requeueing): %s" % (frame, x))
                    self.store.requeue(destination, frame)
                    raise
                
    def _send_frame(self, connection, frame):
        """
        Sends a frame to a specific subscriber connection.
        
        (This method assumes it is being called from within a lock-guarded public
        method.)
        
        @param connection: The subscriber connection object to send to.
        @type connection: L{coilmq.server.StompConnection}
        
        @param frame: The frame to send.
        @type frame: L{stompclient.frame.Frame}
        """
        assert connection is not None
        assert frame is not None
        
        self.log.debug("Delivering frame %s to connection %s" % (frame, connection))
        
        if connection.reliable_subscriber:
            if connection in self._pending:
                raise RuntimeError("Connection already has a pending frame.")
            self.log.debug("Tracking frame %s as pending for connection %s" % (frame, connection))
            self._pending[connection] = frame
            
        connection.send_frame(frame)


# -*- coding: utf-8 -*-
"""
Functional tests that use the default memory-based storage backends and default
scheduler implementations.
"""
import zlib

from coilmq.auth.simple import SimpleAuthenticator
from coilmq.tests.functional import BaseFunctionalTestCase

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class BasicTest(BaseFunctionalTestCase):
    """
    Functional tests using default storage engine, etc.
    """
        
    def test_connect(self):
        """ Test a basic (non-auth) connection. """
        c = self._new_client()
    
    def test_connect_auth(self):
        """ Test connecting when auth is required. """
        self.server.authenticator = SimpleAuthenticator(store={'user': 'pass'})
        
        c1 = self._new_client(connect=False)
        c1.connect()
        r = c1.received_frames.get(timeout=1)
        assert r.command == 'ERROR'
        assert 'Auth' in r.body
        
        c2 = self._new_client(connect=False)
        c2.connect(headers={'login': 'user', 'passcode': 'pass'})
        r2 = c2.received_frames.get(timeout=1)
        print r2
        
        assert r2.command == 'CONNECTED'
        
        c3 = self._new_client(connect=False)
        c3.connect(headers={'login': 'user', 'passcode': 'pass-invalid'})
        r3 = c3.received_frames.get(timeout=1)
        print r3
        
        assert r3.command == 'ERROR'
        
    def test_subscribe(self):
        c1 = self._new_client()
        c1.subscribe('/queue/foo')
        
        c2 = self._new_client()
        c2.subscribe('/queue/foo2')
        
        c2.send('/queue/foo', 'A message')
        assert c2.received_frames.qsize() == 0
       
        r = c1.received_frames.get()
        assert r.command == 'MESSAGE'
        assert r.body == 'A message'
    
    def test_disconnect(self):
        """
        Test the 'polite' disconnect.
        """
        c1 = self._new_client()
        c1.connect()
        c1.disconnect()
        assert c1.received_frames.qsize() == 0
        
    def test_send_binary(self):
        """
        Test sending binary data.
        """
        c1 = self._new_client()
        c1.subscribe('/queue/foo')
        
        # Read some random binary data.
        # (This should be cross-platform.)
        message = 'This is the message that will be compressed.'
        c2 = self._new_client()
        compressed = zlib.compress(message)
        print '%r' % compressed
        c2.send('/queue/foo', zlib.compress(message))
       
        r = c1.received_frames.get()
        assert r.command == 'MESSAGE'
        print '%r' % r.body
        assert zlib.decompress(r.body) == message 
    
    def test_send_utf8(self):
        """
        Test sending utf-8-encoded strings.
        """
        c1 = self._new_client()
        c1.subscribe('/queue/foo')
        
        unicodemsg = u'我能吞下玻璃而不伤身体'
        utf8msg = unicodemsg.encode('utf-8')
        
        print "len(unicodemsg) = %d" % len(unicodemsg)
        print "len(utf8msg) = %d" % len(utf8msg)
        c2 = self._new_client()
        
        print '%r' % utf8msg
        c2.send('/queue/foo', utf8msg)
        
        r = c1.received_frames.get()
        assert r.command == 'MESSAGE'
        print '%r' % r.body
        assert r.body == utf8msg


"""
Configuration support functionality.

The global C{config} object (C{ConfigParser.SafeConfigParser} instance) is initialized
with default configuration from the defaults.cfg file, which is located in this package.
In order to ensure that the config contains custom values, you must call the C{init_config}
function during application initialization:

from coilmq.config import config, init_config
init_config('/path/to/config.cfg')

config.getint('listen_port')
"""
import os.path
import logging
import logging.config
import ConfigParser
import warnings

from pkg_resources import resource_stream

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

config = ConfigParser.SafeConfigParser()
config.readfp(resource_stream(__name__, 'defaults.cfg'))

def init_config(config_file):
    """
    Initialize the configuration from a config file.
    
    The values in config_file will override those already loaded from the default
    configuration file (defaults.cfg, in current package).
    
    This method does not setup logging.
    
    @param config_file: The path to a configuration file.
    @type config_file: C{str}
    
    @raise ValueError: if the specified config_file could not be read.
    @see: L{init_logging}  
    """
    global config
    
    if config_file and os.path.exists(config_file):
        read = config.read([config_file])
        if not read:
            raise ValueError("Could not read configuration from file: %s" % config_file)
        
def init_logging(logfile=None, loglevel=logging.INFO, configfile=None):
    """
    Configures the logging using either basic filename + loglevel or passed config file path.
    
    This is performed separately from L{init_config()} in order to support the case where 
    logging should happen independent of (usu. *after*) other aspects of the configuration 
    initialization. For example, if logging may need to be initialized within a  daemon 
    context.
    
    @param logfile: An explicitly specified logfile destination.  If this is specified in addition
                    to default logging, a warning will be issued.
    @type logfile: C{str}
    
    @param loglevel: Which level to use when logging to explicitly specified file or stdout.
    @type loglevel: C{int}
    
    @param configfile: The path to a configuration file.  This takes precedence over any explicitly
                        specified logfile/loglevel (but a warning will be logged if both are specified).
                        If the file is not specified or does not exist annd no logfile was specified, 
                        then the default.cfg configuration file will be used to initialize logging.
    @type configfile: C{str}
    """
    # If a config file was specified, we will use that in place of the
    # explicitly 
    use_configfile = False
    if configfile and os.path.exists(configfile):
        testcfg = ConfigParser.SafeConfigParser()
        read = testcfg.read(configfile)
        use_configfile = (read and testcfg.has_section('loggers')) 
    
    if use_configfile:
        logging.config.fileConfig(configfile)
        if logfile:
            msg = "Config file conflicts with explicitly specified logfile; config file takes precedence."
            logging.warn(msg)
    else:
        format = '%(asctime)s [%(threadName)s] %(name)s - %(levelname)s - %(message)s'
        if logfile:
            logging.basicConfig(filename=logfile, level=loglevel, format=format)
        else:
            logging.basicConfig(level=loglevel, format=format)
        
def resolve_name(name):
    """
    Resolve a dotted name to some object (usually class, module, or function).
    
    Supported naming formats include:
        1. path.to.module:method
        2. path.to.module.ClassName
    
    >>> resolve_name('coilmq.store.memory.MemoryQueue')
    <class 'coilmq.store.memory.MemoryQueue'>
    >>> t = resolve_name('coilmq.store.dbm.make_dbm')
    >>> type(t)
    <type 'function'>
    >>> t.__name__
    'make_dbm'
    
    @param name: The dotted name (e.g. path.to.MyClass)
    @type name: C{str}
    
    @return: The resolved object (class, callable, etc.) or None if not found.
    """
    if ':' in name:
        # Normalize foo.bar.baz:main to foo.bar.baz.main
        # (since our logic below will handle that)
        name = '%s.%s' % tuple(name.split(':'))
        
    name = name.split('.')
    
    used = name.pop(0)
    found = __import__(used)
    for n in name:
        used = used + '.' + n
        try:
            found = getattr(found, n)
        except AttributeError:
            __import__(used)
            found = getattr(found, n)
            
    return found


"""
Test DBM queue storage.
"""
import unittest
import tempfile
import shutil
import uuid
import time

from stompclient.frame import Frame

from coilmq.store.dbm import DbmQueue

from coilmq.tests.store import CommonQueueTestsMixin

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class DbmQueueTest(unittest.TestCase, CommonQueueTestsMixin):
    
    def setUp(self):
        self.data_dir = tempfile.mkdtemp(prefix='coilmq-dbm-test')
        self.store = DbmQueue(self.data_dir)
    
    def tearDown(self):
        shutil.rmtree(self.data_dir)
        
    def test_dequeue_identity(self):
        """ Test the dequeue() method. """
        dest = '/queue/foo'
        frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data') 
        self.store.enqueue(dest, frame)
        
        assert self.store.has_frames(dest) == True
        assert self.store.size(dest) == 1
        
        rframe = self.store.dequeue(dest)
        assert frame == rframe
        assert frame is not rframe 
        
        assert self.store.has_frames(dest) == False
        assert self.store.size(dest) == 0
    
    def test_sync_checkpoint_ops(self):
        """ Test a expected sync behavior with checkpoint_operations param. """
        
        data_dir = tempfile.mkdtemp(prefix='coilmq-dbm-test')
        max_ops = 5
        try:
            store = DbmQueue(data_dir, checkpoint_operations=max_ops)
            dest = '/queue/foo'
            
            for i in range(max_ops+1):
                frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data - %d' % i)
                store.enqueue(dest, frame)
            
            print store.queue_metadata[dest]    
            assert store.size(dest) == max_ops + 1
            
            # No close()!
            
            store2 = DbmQueue(data_dir)
            print store2.queue_metadata[dest]
            assert store2.size(dest) == max_ops + 1
            
        except:
            shutil.rmtree(data_dir, ignore_errors=True)
            raise
    
    def test_sync_checkpoint_timeout(self):
        """ Test a expected sync behavior with checkpoint_timeout param. """
        
        data_dir = tempfile.mkdtemp(prefix='coilmq-dbm-test')
        try:
            store = DbmQueue(data_dir, checkpoint_timeout=0.5)
            dest = '/queue/foo'
            
            frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data -1')
            store.enqueue(dest, frame)
            
            time.sleep(0.5)
            
            frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data -2')
            store.enqueue(dest, frame)
            
            print store.queue_metadata[dest]    
            assert store.size(dest) == 2
            
            # No close()!
            
            store2 = DbmQueue(data_dir)
            print store2.queue_metadata[dest]
            assert store2.size(dest) == 2
            
        except:
            shutil.rmtree(data_dir, ignore_errors=True)
            raise
        
    def test_sync_close(self):
        """ Test a expected sync behavior of close() call. """
        
        data_dir = tempfile.mkdtemp(prefix='coilmq-dbm-test')
        try:
            store = DbmQueue(data_dir)
            dest = '/queue/foo'
            frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data')
            store.enqueue(dest, frame)
            assert store.size(dest) == 1
            
            store.close()
            
            store2 = DbmQueue(data_dir)
            assert store2.size(dest) == 1
            
        except:
            shutil.rmtree(data_dir, ignore_errors=True)
            raise
        
    def test_sync_loss(self):
        """ Test metadata loss behavior. """
        
        data_dir = tempfile.mkdtemp(prefix='coilmq-dbm-test')
        try:
            store = DbmQueue(data_dir)
            dest = '/queue/foo'
            frame = Frame('MESSAGE', headers={'message-id': str(uuid.uuid4())}, body='some data')
            store.enqueue(dest, frame)
            assert store.size(dest) == 1
            
            store2 = DbmQueue(data_dir)
            assert store2.size(dest) == 0
            
        except:
            shutil.rmtree(data_dir, ignore_errors=True)
            raise


#!python
"""
Entrypoint for starting the application.
"""
import os
import sys
import logging
from optparse import OptionParser

try:
    daemon_support = True
    import daemon
    import signal
    import lockfile
except ImportError:
    daemon_support = False
    
from coilmq.config import config as global_config, init_config, init_logging, resolve_name
from coilmq.topic import TopicManager
from coilmq.queue import QueueManager

from coilmq.exception import ConfigError
from coilmq.server.socketserver import ThreadedStompServer

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

logger = lambda: logging.getLogger(__name__)

def server_from_config(config=None, server_class=None, additional_kwargs=None):
    """
    Gets a configured L{coilmq.server.StompServer} from specified config.
    
    If `config` is None, global L{coilmq.config.config} var will be used instead.
    
    The `server_class` and `additional_kwargs` are primarily hooks for using this method
    from a testing environment.
    
    @param config: A C{ConfigParser.ConfigParser} instance with loaded config values.
    @type config: C{ConfigParser.ConfigParser}
    
    @param server_class: Which class to use for the server.  (This doesn't come from config currently.)
    @type server_class: C{class}
    
    @param additional_kwargs: Any additional args that should be passed to class.
    @type additional_kwargs: C{list}
    
    @return: The configured StompServer.
    @rtype: L{coilmq.server.StompServer}
    """
    global global_config
    if not config: config = global_config
    
    queue_store_factory = resolve_name(config.get('coilmq', 'qstore.factory'))
    subscriber_scheduler_factory = resolve_name(config.get('coilmq', 'scheduler.subscriber_priority_factory'))
    queue_scheduler_factory = resolve_name(config.get('coilmq', 'scheduler.queue_priority_factory'))
    
    if config.has_option('coilmq', 'auth.factory'):
        authenticator_factory = resolve_name(config.get('coilmq', 'auth.factory'))
        authenticator = authenticator_factory()
    else:
        authenticator = None
        
    server = ThreadedStompServer((config.get('coilmq', 'listen_addr'), config.getint('coilmq', 'listen_port')),
                                 queue_manager=QueueManager(store=queue_store_factory(),
                                                            subscriber_scheduler=subscriber_scheduler_factory(),
                                                            queue_scheduler=queue_scheduler_factory()),
                                 topic_manager=TopicManager(),
                                 authenticator=authenticator)
    logger().info("Created server:%r"% server)
    return server


def context_serve(options, context):
    """
    Takes a context object, which implements the __enter__/__exit__ "with" interface 
    and starts a server within that context.
    
    This method is a refactored single-place for handling the server-run code whether
    running in daemon or non-daemon mode.  It is invoked with a dummy (passthrough) 
    context object for the non-daemon use case. 
    
    @param options: The compiled collection of options that need to be parsed. 
    @type options: C{ConfigParser}
    
    @param context: The context object that implements __enter__/__exit__ "with" methods.
    @type context: C{object}
    
    @raise Exception: Any underlying exception will be logged but then re-raised.
    @see: server_from_config()
    """
    server = None
    try:
        with context:
            # There's a possibility here that init_logging() will throw an exception.  If it does,
            # AND we're in a daemon context, then we're not going to be able to do anything with it.
            # We've got no stderr/stdout here; and so (to my knowledge) no reliable (& cross-platform),
            # way to display errors.
            level = logging.DEBUG if options.debug else logging.INFO
            init_logging(logfile=options.logfile, loglevel=level, configfile=options.configfile)
            
            server = server_from_config()
            logger().info("Stomp server listening on %s:%s" % server.server_address)
            server.serve_forever()
    except (KeyboardInterrupt, SystemExit):
        logger().info("Stomp server stopped by user interrupt.")
        raise SystemExit()
    except Exception, e:
        logger().error("Stomp server stopped due to error: %s" % e)
        logger().exception(e)
        raise SystemExit()
    finally:
        if server: server.server_close()
    
def main():
    """
    Main entry point for running a socket server from the commandline.
    
    This method will read in options from the commandline and call the L{config.init_config} method
    to get everything setup.  Then, depending on whether deamon mode was specified or not, 
    the process may be forked (or not) and the server will be started.
    """

    parser = OptionParser()
    parser.add_option("-c", "--config", dest="configfile",
                      help="Read configuration from FILE. (CLI options override config file.)", metavar="FILE")
    
    parser.add_option("-b", "--host", dest="listen_addr",
                      help="Listen on specified address (default 0.0.0.0)", metavar="ADDR")
    
    parser.add_option("-p", "--port", dest="listen_port",
                      help="Listen on specified port (default 61613)", type="int", metavar="PORT")

    parser.add_option("-l", "--logfile", dest="logfile",
                      help="Log to specified file (unless logging configured in config file).", metavar="FILE")

    parser.add_option("--debug", action="store_true", dest="debug", default=False, 
                      help="Sets logging to debug (unless logging configured in config file).")
    
    parser.add_option("-d", "--daemon", action="store_true", dest="daemon", default=False,
                      help="Run server as a daemon (default False).")
    
    parser.add_option("-u", "--uid", dest="uid",
                      help="The user/UID to use for daemon process.", metavar="UID")
    
    parser.add_option("-g", "--gid", dest="gid",
                      help="The group/GID to use for daemon process.", metavar="GID")
    
    parser.add_option("--pidfile", dest="pidfile", help="The PID file to use.", metavar="FILE")
    
    parser.add_option("--umask", dest="umask",
                      help="Umask (octal) to apply for daemonized process.", metavar="MASK")
    
    parser.add_option('--rundir', dest="rundir",
                      help="The working directory to use for the daemonized process (default /).", metavar="DIR")
    
    (options, args) = parser.parse_args()
    
    # Note that we must initialize the configuration before we enter the context
    # block; however, we _cannot_ initialize logging until we are in the context block
    # (so we defer that until the context_serve call.)
    init_config(options.configfile)
    
    if options.listen_addr is not None:
        global_config.set('coilmq', 'listen_addr', options.listen_addr)
        
    if options.listen_port is not None:
        global_config.set('coilmq', 'listen_port', str(options.listen_port))
    
    if options.daemon:
        if not daemon_support:
            raise ConfigError("This environment/platform does not support running as daemon. (Are you running on *nix and did you install python-daemon package?)")
        else:
            
            context = daemon.DaemonContext()
            
            if options.uid is not None:
                context.uid = options.uid
                
            if options.gid is not None:
                context.gid = options.gid
                
            if options.pidfile is not None:
                context.pidfile = lockfile.FileLock(options.pidfile)
            
            if options.umask is not None:
                context.umask = int(options.umask, 8)
            
            if options.rundir is not None:
                context.working_directory = options.rundir
                
            context_serve(options, context)

    else:
        # Non-daemon mode, so we use a dummy context objectx
        # so we can use the same run-server code as the daemon version.
        
        class DumbContext(object):
            def __enter__(self):
                pass
            def __exit__(self, type, value, traceback):
                pass
        
        context_serve(options, DumbContext())
        
if __name__ == '__main__':
    try:
        main()
    except (KeyboardInterrupt, SystemExit):
        pass
    except Exception, e:
        logger().error("Server terminated due to error: %s" % e)
        logger().exception(e)


"""
Queue storage module that stores the queue information and frames in a DBM-style database.

The current implementation uses the Python `shelve` module, which uses a DBM implementation
under the hood (specifically the `anydbm` module, aka `dbm` in Python 3.x).

Because of how the `shelve` module works (and how we're using it) and caveats in the Python 
documentation this is likely a BAD storage module to use if you are expecting to traffic in
large frames.
"""
import threading
import logging
import os
import os.path
import shelve
import ConfigParser
from collections import deque
from datetime import datetime, timedelta


from coilmq.store import QueueStore
from coilmq.config import config
from coilmq.exception import ConfigError
from coilmq.util.concurrency import synchronized

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

def make_dbm():
    """
    Creates a DBM queue store, pulling config values from the CoilMQ configuration.
    """
    try:
        data_dir = config.get('coilmq', 'qstore.dbm.data_dir')
        cp_ops = config.getint('coilmq', 'qstore.dbm.checkpoint_operations')
        cp_timeout = config.getint('coilmq', 'qstore.dbm.checkpoint_timeout')
    except ConfigParser.NoOptionError, e:
        raise ConfigError('Missing configuration parameter: %s' % e)
    
    if not os.path.exists(data_dir):
        raise ConfigError('DBM directory does not exist: %s' % data_dir)
    if not os.access(data_dir, os.W_OK | os.R_OK): # FIXME: how do these get applied? Is OR appropriate?
        raise ConfigError('Cannot read and write DBM directory: %s' % data_dir)
    
    store = DbmQueue(data_dir, checkpoint_operations=cp_ops, checkpoint_timeout=cp_timeout)
    return store

class DbmQueue(QueueStore):
    """
    A QueueStore implementation that stores messages and queue information in DBM-style
    database.
    
    Several database files will be used to support this functionality: metadata about the
    queues will be stored in its own database and each queue will also have its own
    database file.
    
    This classes uses a C{threading.RLock} to guard access to the memory store, since it
    appears that at least some of the underlying implementations that anydbm uses are not
    thread-safe
    
    Due to some impedence mismatch between the types of data we need to store in queues
    (specifically lists) and the types of data that are best stored in DBM databases
    (specifically dicts), this class uses the `shelve` module to abstract away some
    of the ugliness.  The consequence of this is that we only persist objects periodically
    to the datastore, for performance reasons.  How periodic is determined by the 
    `checkpoint_operations` and `checkpoint_timeout` instance variables (and params to 
    L{__init__}).
    
    @ivar data_dir: The directory where DBM files will be stored.
    @type data_dir: C{str}
    
    @ivar queue_metadata: A Shelf (DBM) database that tracks stats & delivered message ids 
                            for all the queues.
    @ivar queue_metadata: C{shelve.Shelf}
    
    @ivar frame_store: A Shelf (DBM) database that contains frame contents indexed by message id.
    @type frame_store: C{shelve.Shelf}
    
    @ivar _opcount: Internal counter for keeping track of unpersisted operations.
    @type _opcount: C{int}
    
    @ivar checkpoint_operations: Number of operations between syncs.
    @type checkpoint_operations: C{int}
    
    @ivar checkpoint_timeout: Max time (in seconds) that can elapse between sync of cache.
    @type checkpoint_timeout: C{float}
    """
    def __init__(self, data_dir, checkpoint_operations=100, checkpoint_timeout=30):
        """
        @param data_dir: The directory where DBM files will be stored.
        @param data_dir: C{str}
        
        @param checkpoint_operations: Number of operations between syncs.
        @type checkpoint_operations: C{int}
    
        @param checkpoint_timeout: Max time (in seconds) that can elapse between sync of cache.
        @type checkpoint_timeout: C{float}
        """
        QueueStore.__init__(self)
        
        self._opcount = 0
        self._last_sync = datetime.now()
        
        self.data_dir = data_dir
        self.checkpoint_operations = checkpoint_operations
        self.checkpoint_timeout = timedelta(seconds=checkpoint_timeout)
        
        # Should this be in constructor?
        
        # The queue metadata stores mutable (dict) objects.  For this reason we set
        # writeback=True and rely on the sync() method to keep the cache & disk in-sync.
        self.queue_metadata = shelve.open(os.path.join(self.data_dir, 'metadata'), writeback=True)
        
        # Since we do not need mutable objects on the frame stores (we don't modify them, we just
        # put/get values), we do NOT use writeback=True here.  This should also conserve on memory
        # usage, since apparently that can get hefty with the caching when writeback=True.
        self.frame_store = shelve.open(os.path.join(self.data_dir, 'frames'), writeback=False)
    
    @synchronized
    def enqueue(self, destination, frame):
        """
        Store message (frame) for specified destinationination.
        
        @param destination: The destinationination queue name for this message (frame).
        @type destination: C{str}
        
        @param frame: The message (frame) to send to specified destinationination.
        @type frame: C{stompclient.frame.Frame}
        """
        message_id = frame.message_id
        if not message_id:
            raise ValueError("Cannot queue a frame without message-id set.")
        
        if not destination in self.queue_metadata:
            self.log.info("Destination %s not in metadata; creating new entry and queue database." % destination)
            self.queue_metadata[destination] = {'frames': deque(), 'enqueued': 0, 'dequeued': 0, 'size': 0}
            
        self.queue_metadata[destination]['frames'].appendleft(message_id)
        self.queue_metadata[destination]['enqueued'] += 1
        
        self.frame_store[message_id] = frame
        
        self._opcount += 1
        self._sync()
    
    @synchronized
    def dequeue(self, destination):
        """
        Removes and returns an item from the queue (or C{None} if no items in queue).
        
        @param destination: The queue name (destinationination).
        @type destination: C{str}
        
        @return: The first frame in the specified queue, or C{None} if there are none.
        @rtype: C{stompclient.frame.Frame} 
        """
        if not self.has_frames(destination):
            return None

        message_id = self.queue_metadata[destination]['frames'].pop()
        self.queue_metadata[destination]['dequeued'] += 1
        
        frame = self.frame_store[message_id]
        del self.frame_store[message_id]
        
        self._opcount += 1
        self._sync()
        
        return frame
    
    @synchronized
    def has_frames(self, destination):
        """
        Whether specified queue has any frames.
        
        @param destination: The queue name (destinationination).
        @type destination: C{str}
        
        @return: Whether there are any frames in the specified queue.
        @rtype: C{bool}
        """
        return (destination in self.queue_metadata) and bool(self.queue_metadata[destination]['frames'])

    @synchronized
    def size(self, destination):
        """
        Size of the queue for specified destination.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        
        @return: The number of frames in specified queue.
        @rtype: C{int}
        """
        if not destination in self.queue_metadata:
            return 0
        else:
            return len(self.queue_metadata[destination]['frames'])
    
    @synchronized
    def close(self):
        """
        Closes the databases, freeing any resources (and flushing any unsaved changes to disk).
        """
        self.queue_metadata.close()
        self.frame_store.close()
            
    def _sync(self):
        """
        Synchronize the cached data with the underlyind database.
        
        Uses an internal transaction counter and compares to the checkpoint_operations
        and checkpoint_timeout paramters to determine whether to persist the memory store.
        
        In this implementation, this method wraps calls to C{shelve.Shelf#sync}. 
        """
        if (self._opcount > self.checkpoint_operations or 
                datetime.now() > self._last_sync + self.checkpoint_timeout):
            self.log.debug("Synchronizing queue metadata.")
            self.queue_metadata.sync()
            self._last_sync = datetime.now()
            self._opcount = 0
        else:
            self.log.debug("NOT synchronizing queue metadata.")
        


"""
Classes that provide delivery scheduler implementations.

The default implementation used by the system for determining which subscriber
(connection) should receive a message is simply a random choice but favoring
reliable subscribers.  Developers can write their own delivery schedulers, which 
should implement the methods defined in L{QueuePriorityScheduler} if they would
like to customize the behavior.
"""
import abc
import random

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class SubscriberPriorityScheduler(object):
    """ Abstract base class for choosing which recipient (subscriber) should receive a message. """
    __metaclass__ = abc.ABCMeta
    
    @abc.abstractmethod
    def choice(self, subscribers, message):
        """
        Chooses which subscriber (from list) should recieve specified message.
        
        @param subscribers: Collection of subscribed connections eligible to receive message. 
        @type subscribers: C{list} of L{coilmq.server.StompConnection}
        
        @param message: The message to be delivered. 
        @type message: L{stompclient.frame.Frame}
        
        @return: A selected subscriber from the list or None if no subscriber could be chosen (e.g. list is empty).
        @rtype: L{coilmq.server.StompConnection}
        """        

class QueuePriorityScheduler(object):
    """
    Abstract base class for objects that provide a way to prioritize the queues.
    """
    
    def choice(self, queues, connection):
        """
        Choose which queue to select for messages to specified connection.
        
        @param queues: A C{dict} mapping queue name to queues (sets of frames) to which 
                        specified connection is subscribed.
        @type queues:  C{dict} of C{str} to C{set} of L{stompclient.frame.Frame}
                
        @param connection: The connection that is going to be delivered the frame(s).
        @type connection: L{coilmq.server.StompConnection}
        
        @return: A selected queue destination (name) or None if queues C{dict} is empty.
        @rtype: C{str}
        """
        raise NotImplementedError()
    
class RandomSubscriberScheduler(SubscriberPriorityScheduler):
    """ A delivery scheduler that chooses a random subscriber for message recipient. """
    
    def choice(self, subscribers, message):
        """
        Chooses a random connection from subscribers to deliver specified message.
        
        @param subscribers: Collection of subscribed connections to destination. 
        @type subscribers: C{list} of L{coilmq.server.StompConnection}
        
        @param message: The message to be delivered. 
        @type message: L{stompclient.frame.Frame}
        
        @return: A random subscriber from the list or None if list is empty.
        @rtype: L{coilmq.server.StompConnection}
        """
        if not subscribers:
            return None
        return random.choice(subscribers)

class FavorReliableSubscriberScheduler(SubscriberPriorityScheduler):
    """
    A random delivery scheduler which prefers reliable subscribers.
    """
    
    def choice(self, subscribers, message):
        """
        Choose a random connection, favoring those that are reliable from
        subscriber pool to deliver specified message.
        
        @param subscribers: Collection of subscribed connections to destination. 
        @type subscribers: C{list} of L{coilmq.server.StompConnection}
        
        @param message: The message to be delivered. 
        @type message: L{stompclient.frame.Frame}
        
        @return: A random subscriber from the list or None if list is empty.
        @rtype: L{coilmq.server.StompConnection}
        """
        if not subscribers:
            return None
        reliable_subscribers = [s for s in subscribers if s.reliable_subscriber]
        if reliable_subscribers:
            return random.choice(reliable_subscribers)
        else:
            return random.choice(subscribers)

class RandomQueueScheduler(QueuePriorityScheduler):
    """
    Implementation of L{QueuePriorityScheduler} that selects a random queue from the list.
    """
    
    def choice(self, queues, connection):
        """
        Chooses a random queue for messages to specified connection.
        
        @param queues: A C{dict} mapping queue name to queues (sets of frames) to which 
                        specified connection is subscribed.
        @type queues:  C{dict} of C{str} to C{set} of L{stompclient.frame.Frame}
        
        @param connection: The connection that is going to be delivered the frame(s).
        @type connection: L{coilmq.server.StompConnection}
        
        @return: A random queue destination or None if list is empty.
        @rtype: C{str}
        """
        if not queues:
            return None
        return random.choice(queues.keys())


"""
Functional tests to test full stack (but not actual socket layer).
"""
import sys
import time
import unittest
import logging
import socket
import select
import threading
from Queue import Queue

from stompclient.frame import Frame
from stompclient.util import FrameBuffer

from coilmq.queue import QueueManager
from coilmq.topic import TopicManager

from coilmq.server.socketserver import StompServer, StompRequestHandler, ThreadedStompServer
from coilmq.store.memory import MemoryQueue
from coilmq.scheduler import FavorReliableSubscriberScheduler, RandomQueueScheduler

from coilmq.tests import mock

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class BaseFunctionalTestCase(unittest.TestCase):
    """
    Base class for test cases provides the fixtures for setting up the multi-threaded
    unit test infrastructure.
    
    We use a combination of C{threading.Event} and C{Queue.Queue} objects to faciliate
    inter-thread communication and lock-stepping the assertions. 
    """
    def setUp(self):
        
        self.clients = []
        self.server = None  # This gets set in the server thread.
        self.server_address = None # This gets set in the server thread.
        self.ready_event = threading.Event()
        
        addr_bound = threading.Event()
        def start_server():
            self.server = TestStompServer(('127.0.0.1', 0),
                                          ready_event=self.ready_event,
                                          authenticator=None,
                                          queue_manager=self._queuemanager(),
                                          topic_manager=self._topicmanager())
            self.server_address = self.server.socket.getsockname()
            addr_bound.set()
            self.server.serve_forever()
            
        self.server_thread = threading.Thread(target=start_server, name='server')
        self.server_thread.start()
        self.ready_event.wait()
        addr_bound.wait()
    
    def _queuemanager(self):
        """
        Returns the configured L{QueueManager} instance to use.
        
        Can be overridden by subclasses that wish to change out any queue mgr parameters.
        
        @rtype: L{QueueManager}
        """
        return QueueManager(store=MemoryQueue(),
                            subscriber_scheduler=FavorReliableSubscriberScheduler(),
                            queue_scheduler=RandomQueueScheduler())
    
    def _topicmanager(self):
        """
        Returns the configured L{TopicManager} instance to use.
        
        Can be overridden by subclasses that wish to change out any topic mgr parameters.
        
        @rtype: L{TopicManager}
        """
        return TopicManager()
    
    def tearDown(self):
        for c in self.clients:
            print "Disconnecting %s" % c
            c.close()
        self.server.shutdown()
        self.server_thread.join()
        self.ready_event.clear()
        del self.server_thread
        
    def _new_client(self, connect=True):
        """
        Get a new L{TestStompClient} connected to our test server. 
        
        The client will also be registered for close in the tearDown method.
        
        @param connect: Whether to issue the CONNECT command.
        @type connect: C{bool}
        
        @rtype: L{TestStompClient}
        """
        client = TestStompClient(self.server_address)
        self.clients.append(client)
        if connect:
            client.connect()
            r = client.received_frames.get(timeout=1)
            assert r.command == 'CONNECTED'
        return client

class TestStompServer(ThreadedStompServer):
    """
    A stomp server for functional tests that uses C{threading.Event} objects
    to ensure that it stays in sync with the test suite.
    """
    
    allow_reuse_address = True
    
    def __init__(self, server_address,
                 ready_event=None,
                 authenticator=None,
                 queue_manager=None,
                 topic_manager=None):
        self.ready_event = ready_event
        StompServer.__init__(self, server_address, StompRequestHandler,
                             authenticator=authenticator,
                             queue_manager=queue_manager,
                             topic_manager=topic_manager)
        
    def server_activate(self):
        self.ready_event.set()
        StompServer.server_activate(self)

class TestStompClient(object):
    """
    A stomp client for use in testing.
    
    This client spawns a listener thread and pushes anything that comes in onto the 
    read_frames queue.
    
    @ivar received_frames: A queue of Frame instances that have been received.
    @type received_frames: C{Queue.Queue} containing any received C{stompclient.frame.Frame}
    """
    def __init__(self, addr, connect=True):
        """
        @param addr: The (host,port) tuple for connection.
        @type addr: C{tuple}
        
        @param connect: Whether to connect socket to specified addr.
        @type connect: C{bool}
        """
        self.log = logging.getLogger('%s.%s' % (self.__module__, self.__class__.__name__))
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.addr = addr
        self.received_frames = Queue()
        self.read_stopped = threading.Event()
        self.buffer = FrameBuffer()
        if connect:
            self._connect()
    
    def connect(self, headers=None):
        if headers is None:
            headers = {}
        self.send_frame(Frame('CONNECT', headers=headers))
        
    def send(self, destination, message, set_content_length=True):
        headers = {'destination': destination}
        if set_content_length: headers['content-length'] = len(message)
        self.send_frame(Frame('SEND', headers=headers, body=message))
    
    def subscribe(self, destination):
        self.send_frame(Frame('SUBSCRIBE', headers={'destination': destination}))
        
    def send_frame(self, frame):
        """
        Sends a stomp frame.
        @param frame: The stomp frame to send.
        @type frame: L{stompclient.frame.Frame}
        """
        if not self.connected:
            raise RuntimeError("Not connected")
        self.sock.send(frame.pack())
    
    def _connect(self):
        self.sock.connect(self.addr)
        self.connected = True
        self.read_stopped.clear()
        t = threading.Thread(target=self._read_loop, name="client-receiver-%s" % hex(id(self)))
        t.start()
    
    def _read_loop(self):
        while self.connected:
            r, w, e = select.select([self.sock], [], [], 0.1)
            if r:
                data = self.sock.recv(1024)
                self.log.debug("Data received: %r" % data)
                self.buffer.append(data)
                for frame in self.buffer:
                    self.log.debug("Processing frame: %s" % frame)
                    self.received_frames.put(frame)
        self.read_stopped.set()
        # print "Read loop has been quit! for %s" % id(self)
    
    def disconnect(self):
        self.send_frame(Frame('DISCONNECT'))
        
    def close(self):
        if not self.connected:
            raise RuntimeError("Not connected")
        self.connected = False
        self.read_stopped.wait()
        self.sock.close()



"""
Queue storage module that uses SQLAlchemy to access queue information and frames in a database.


"""
import threading
import logging
import os
import os.path
import shelve
import ConfigParser
from collections import deque
from datetime import datetime, timedelta

from sqlalchemy import engine_from_config, MetaData
from sqlalchemy.orm import scoped_session, sessionmaker
from sqlalchemy.sql import select, func

from coilmq.store import QueueStore
from coilmq.config import config
from coilmq.exception import ConfigError
from coilmq.util.concurrency import synchronized
from coilmq.store.sa import meta, model

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

def make_sa():
    """
    Factory to creates a SQLAlchemy queue store, pulling config values from the CoilMQ configuration.
    """
    configuration = dict(config.items('coilmq'))
    engine = engine_from_config(configuration, 'qstore.sqlalchemy.')
    init_model(engine)
    store = SAQueue()
    return store

def init_model(engine):
    """
    Initializes the shared SQLAlchemy state in the L{coilmq.store.sa.model} module.
    
    @param engine: The SQLAlchemy engine instance.
    @type engine: C{sqlalchemy.Engine}
    """
    meta.engine = engine
    meta.metadata = MetaData(bind=meta.engine)
    meta.Session = scoped_session(sessionmaker(bind=meta.engine))
    model.setup_tables()
    
class SAQueue(QueueStore):
    """
    A QueueStore implementation that stores messages in a database and uses SQLAlchemy to interface
    with the database.
    
    Note that this implementation does not actually use the ORM capabilities of SQLAlchemy, but simply
    uses SQLAlchemy for the DB abstraction for SQL building and DDL (table creation).
    
    This L{coilmq.store.sa.model.setup_tables} function is used to actually define (& create) the 
    database tables.  This class also depends on the L{init_model} method have been called to 
    define the L{coilmq.store.sa.model.Session} class-like callable (and the engine & metadata).
     
    Finally, this class does not explicitly use shared data (db connections); a new Session is created
    in each method.  The actual implementation is handled using SQLAlchemy scoped sessions, which provide
    thread-local Session class-like callables. As a result of deferring that to the SA layer, we don't 
    need to use synchronization locks to guard calls to the methods in this store implementation.
    """
        
    def enqueue(self, destination, frame):
        """
        Store message (frame) for specified destinationination.
        
        @param destination: The destinationination queue name for this message (frame).
        @type destination: C{str}
        
        @param frame: The message (frame) to send to specified destinationination.
        @type frame: C{stompclient.frame.Frame}
        """
        session = meta.Session()
        message_id = frame.message_id
        if not message_id:
            raise ValueError("Cannot queue a frame without message-id set.")
        ins = model.frames_table.insert().values(message_id=frame.message_id, destination=destination, frame=frame)
        session.execute(ins)
        session.commit()
    
    def dequeue(self, destination):
        """
        Removes and returns an item from the queue (or C{None} if no items in queue).
        
        @param destination: The queue name (destinationination).
        @type destination: C{str}
        
        @return: The first frame in the specified queue, or C{None} if there are none.
        @rtype: C{stompclient.frame.Frame} 
        """
        session = meta.Session()
        
        try:
            
            selstmt = select([model.frames_table.c.message_id, model.frames_table.c.frame])
            selstmt = selstmt.where(model.frames_table.c.destination==destination)
            selstmt = selstmt.order_by(model.frames_table.c.queued, model.frames_table.c.sequence)
            
            result = session.execute(selstmt)
            
            first = result.fetchone()
            if not first:
                return None
            
            delstmt = model.frames_table.delete().where(model.frames_table.c.message_id==first[model.frames_table.c.message_id])
            session.execute(delstmt) 
            
            frame = first[model.frames_table.c.frame] 
            
        except:
            session.rollback()
            raise
        else:
            session.commit()
            return frame
            
    def has_frames(self, destination):
        """
        Whether specified queue has any frames.
        
        @param destination: The queue name (destinationination).
        @type destination: C{str}
        
        @return: Whether there are any frames in the specified queue.
        @rtype: C{bool}
        """
        session = meta.Session()
        sel = select([model.frames_table.c.message_id]).where(model.frames_table.c.destination==destination)
        result = session.execute(sel)
        
        first = result.fetchone()
        print first
        return first is not None

    def size(self, destination):
        """
        Size of the queue for specified destination.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        
        @return: The number of frames in specified queue.
        @rtype: C{int}
        """
        session = meta.Session()
        sel = select([func.count(model.frames_table.c.message_id)]).where(model.frames_table.c.destination==destination)
        result = session.execute(sel)
        first = result.fetchone()
        if not first:
            return 0
        else:
            return int(first[0])
            
    def close(self):
        """
        Closes the databases, freeing any resources (and flushing any unsaved changes to disk).
        """
        meta.Session.remove()


"""
Core STOMP server logic, abstracted from socket transport implementation.

While this is abstracted from the socket transport implementation, it does
operate on the assumption that there is an available connection to send response
frames.

We're also making some simplified assumptions here which may make this engine
impractical for [high-performance] use in specifically asynchronous frameworks.
More specifically, this class was not explicitly designed around async patterns, 
meaning that it would likely be problematic to use with a framework like Twisted 
if the underlying storage implementations were processor intensive (e.g. database
access).  For the default memory storage engines, this shouldn't be a problem.

This code is inspired by the design of the Ruby stompserver project, by 
Patrick Hurley and Lionel Bouton.  See http://stompserver.rubyforge.org/
"""
import logging
import uuid
from collections import defaultdict

from stompclient.frame import ConnectedFrame, ReceiptFrame, ErrorFrame

from coilmq.exception import ProtocolError, AuthError

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

VALID_COMMANDS = ['connect', 'send', 'subscribe', 'unsubscribe', 'begin', 'commit', 'abort', 'ack', 'disconnect']

class StompEngine(object):
    """ 
    The engine provides the core business logic that we use to respond to STOMP protocol
    messages.  
    
    This class is transport-agnostic; it exposes methods that expect STOMP frames and
    uses the attached connection to send frames to connected clients.
    
    @ivar connection: The connection (aka "protocol") backing this engine.
    @type connection: L{coilmq.server.StompConnection}
    
    @ivar authenticator: An C{Authenticator} implementation to use.  Setting this value
                            will implicitly cause authentication to be required.
    @type authenticator: L{coilmq.auth.Authenticator}
    
    @ivar queue_manager: The C{QueueManager} implementation to use.
    @type queue_manager: L{coilmq.queue.QueueManager}
    
    @ivar topic_manager: The C{TopicManager} implementation to use.
    @type topic_manager: L{coilmq.topic.TopicManager}
    
    @ivar transactions: Active transactions for this connection.
    @type transactions: C{dict} of C{str} to C{list} 
    
    @ivar connected: Whether engine is connected.
    @type connected: C{bool}
    """
    def __init__(self, connection, authenticator, queue_manager, topic_manager):
        """
        @param connection: The stomp connection backing this engine.
        @type connection: L{coilmq.server.StompConnection}
        """
        self.log = logging.getLogger('%s.%s' % (self.__class__.__module__, self.__class__.__name__))
        self.connection = connection
        self.authenticator = authenticator
        self.queue_manager = queue_manager
        self.topic_manager = topic_manager
        self.connected = False
        self.transactions = defaultdict(list)
    
    def process_frame(self, frame):
        """
        Dispatches a received frame to the appropriate internal method.
        
        @param frame: The frame that was received.
        @type frame: C{stompclient.frame.Frame} 
        """
        cmd_method = frame.command.lower()
        
        if not cmd_method in VALID_COMMANDS:
            raise ProtocolError("Invalid STOMP command: %s" % frame.command)
        
        method = getattr(self, cmd_method, None)
        
        if not self.connected and method != self.connect:
            raise ProtocolError("Not connected.")
        
        try:
            transaction = frame.headers.get('transaction')
            if not transaction or method in (self.begin, self.commit, self.abort):
                method(frame) 
            else:
                if not transaction in self.transactions:
                    raise ProtocolError("Invalid transaction specified: %s" % transaction)
                self.transactions[transaction].append(frame) 
        except Exception, e:
            self.log.error("Error processing STOMP frame: %s" % e)
            self.log.exception(e)
            try:
                self.connection.send_frame(ErrorFrame(str(e), str(e)))
            except Exception, e:
                self.log.error("Could not send error frame: %s" % e)
                self.log.exception(e)
        
    def connect(self, frame):
        """
        Handle CONNECT command: Establishes a new connection and checks auth (if applicable).
        """
        self.log.debug("CONNECT")
        
        if self.authenticator:
            login = frame.headers.get('login')
            passcode = frame.headers.get('passcode')
            if not self.authenticator.authenticate(login, passcode):
                raise AuthError("Authentication failed for %s" % login)
        
        self.connected = True
        
        # TODO: Do we want to do anything special to track sessions?
        # (Actually, I don't think the spec actually does anything with this at all.)
        self.connection.send_frame(ConnectedFrame(session=uuid.uuid4()))
    
    def send(self, frame):
        """
        Handle the SEND command: Delivers a message to a queue or topic (default).
        """
        dest = frame.headers.get('destination')
        if not dest:
            raise ProtocolError('Missing destination for SEND command.')
        
        if dest.startswith('/queue/'):
            self.queue_manager.send(frame)
        else:
            self.topic_manager.send(frame)
            
    def subscribe(self, frame):
        """
        Handle the SUBSCRIBE command: Adds this connection to destination.
        """
        ack = frame.headers.get('ack')
        reliable = ack and ack.lower() == 'client'
        
        self.connection.reliable_subscriber = reliable
        
        dest = frame.headers.get('destination')
        if not dest:
            raise ProtocolError('Missing destination for SUBSCRIBE command.')
        
        if dest.startswith('/queue/'):
            self.queue_manager.subscribe(self.connection, dest)
        else:
            self.topic_manager.subscribe(self.connection, dest)
            
    def unsubscribe(self, frame):
        """
        Handle the UNSUBSCRIBE command: Removes this connection from destination.
        """
        dest = frame.headers.get('destination')
        if not dest:
            raise ProtocolError('Missing destination for UNSUBSCRIBE command.')
        
        if dest.startswith('/queue/'):
            self.queue_manager.unsubscribe(self.connection, dest)
        else:
            self.topic_manager.unsubscribe(self.connection, dest)
    
    def begin(self, frame):
        """
        Handles BEGING command: Starts a new transaction.
        """
        if not frame.transaction:
            raise ProtocolError("Missing transaction for BEGIN command.")
        
        self.transactions[frame.transaction] = []
    
    def commit(self, frame):
        """
        Handles COMMIT command: Commits specified transaction.
        """
        if not frame.transaction:
            raise ProtocolError("Missing transaction for COMMIT command.")
        
        if not frame.transaction in self.transactions:
            raise ProtocolError("Invalid transaction: %s" % frame.transaction)
        
        for tframe in self.transactions[frame.transaction]:
            del tframe.headers['transaction']
            print "Processing frame: %s" % tframe
            self.process_frame(tframe)
        
        self.queue_manager.clearTransactionFrames(self.connection, frame.transaction)
        del self.transactions[frame.transaction]
    
    def abort(self, frame):
        """
        Handles ABORT command: Rolls back specified transaction.
        """
        if not frame.transaction:
            raise ProtocolError("Missing transaction for ABORT command.")
        
        if not frame.transaction in self.transactions:
            raise ProtocolError("Invalid transaction: %s" % frame.transaction)
        
        self.queue_manager.resend_transaction_frames(self.connection, frame.transaction)
        del self.transactions[frame.transaction]
        
    def ack(self, frame):
        """
        Handles the ACK command: Acknowledges receipt of a message.
        """
        if not frame.message_id:
            raise ProtocolError("No message-id specified for ACK command.")
        self.queue_manager.ack(self.connection, frame)
        
    def disconnect(self, frame):
        """
        Handles the DISCONNECT command: Unbinds the connection. 
        
        Clients are supposed to send this command, but in practice it should not be
        relied upon.
        """
        self.log.debug("Disconnect")
        self.unbind()
    
    def unbind(self):
        """
        Unbinds this connection from queue and topic managers (freeing up resources)
        and resets state.
        """
        self.connected = False
        self.queue_manager.disconnect(self.connection)
        self.topic_manager.disconnect(self.connection)
    


"""
A simple config-file based authentication module.
"""
from ConfigParser import RawConfigParser
from coilmq.auth import Authenticator
from coilmq.config import config
from coilmq.exception import ConfigError

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

def make_simple():
    """
    Create a L{SimpleAuthenticator} instance using values read from coilmq configuration.
    
    @return: The configured L{SimpleAuthenticator}
    @rtype: L{SimpleAuthenticator}
    @raise ConfigError: If there is a configuration error. 
    """
    authfile = config.get('coilmq', 'auth.simple.file')
    if not authfile:
        raise ConfigError('Missing configuration parameter: auth.simple.file')
    sa = SimpleAuthenticator()
    sa.from_configfile(authfile)
    return sa
    
class SimpleAuthenticator(Authenticator):
    """
    A simple configfile-based authenticator.
    
    @ivar store:  Authentication key-value store (of logins to passwords).
    @type store: C{dict} of C{str} to C{str}
    """
    def __init__(self, store=None):
        """
        Initialize the authenticator to use (optionally) specified C{dict} store.
        
        @param store:  Authentication store, C{dict} of logins to passwords.
        @type store: C{dict} of C{str} to C{str}
        """
        if store is None:
            store = {}
        self.store = store
    
    def from_configfile(self, configfile):
        """
        Initialize the authentication store from a "config"-style file.
        
        Auth "config" file is parsed with C{ConfigParser.RawConfigParser} and must contain
        an [auth] section which contains the usernames (keys) and passwords (values).
        
        Example auth file::
        
            [auth]
            someuser = somepass
            anotheruser = anotherpass
        
        @param configfile: Path to config file or file-like object. 
        @type configfile: C{any}
        @raise ValueError: If file could not be read or does not contain [auth] section.
        """
        cfg = RawConfigParser()
        if hasattr(configfile, 'read'):
            cfg.readfp(configfile)
        else:
            filesread = cfg.read(configfile)
            if not filesread:
                raise ValueError('Could not parse auth file: %s' % configfile)
            
        if not cfg.has_section('auth'):
            raise ValueError('Config file contains no [auth] section.')
        
        self.store = dict(cfg.items('auth'))
        
    def authenticate(self, login, passcode):
        """
        Authenticate the login and passcode.
         
        @return: Whether provided login and password match values in store.
        @rtype: C{bool} 
        """
        return login in self.store and self.store[login] == passcode
    


"""
Tests for queue-related classes.
"""
import unittest
import uuid

from stompclient.frame import Frame

from coilmq.queue import QueueManager
from coilmq.store.memory import MemoryQueue
 
from coilmq.tests.mock import MockConnection

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class QueueManagerTest(unittest.TestCase):
    """ Test the QueueManager class. """
    
    def setUp(self):
        self.store = MemoryQueue() 
        self.qm = QueueManager(self.store)
        self.conn = MockConnection()
    
    def test_subscribe(self):
        """ Test subscribing a connection to the queue. """
        dest = '/queue/dest'
        
        self.qm.subscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.qm.send(f)
        
        print self.conn.frames
        assert len(self.conn.frames) == 1
        assert self.conn.frames[0] == f
    
    def test_unsubscribe(self):
        """ Test unsubscribing a connection from the queue. """
        dest = '/queue/dest'
        
        self.qm.subscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.qm.send(f)
        
        print self.conn.frames
        assert len(self.conn.frames) == 1
        assert self.conn.frames[0] == f
        
        self.qm.unsubscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.qm.send(f)
        
        assert len(self.conn.frames) == 1
        assert len(self.store.frames(dest)) == 1
        
    def send_simple(self):
        """ Test a basic send command. """
        dest = '/queue/dest'
        
        f = Frame('SEND', headers={'destination': dest}, body='Empty')
        self.qm.send(f)
        
        assert len(self.store.frames(dest)) == 1
        
        # Assert some side-effects
        assert 'message-id' in f.headers
        assert f.command == 'MESSAGE'
        
    
    def test_send_err(self):
        """ Test sending a message when delivery results in error. """
        
        class ExcThrowingConn(object):
            reliable_subscriber = True
            def send_frame(self, frame):
                raise RuntimeError("Error sending data.")
        
        dest = '/queue/dest'
        
        # This reliable subscriber will be chosen first
        conn = ExcThrowingConn()
        self.qm.subscribe(conn, dest)
        
        f = Frame('SEND', headers={'destination': dest}, body='Empty')
        try:
            self.qm.send(f)
            self.fail("Expected failure when there was an error sending.")
        except RuntimeError:
            pass
        
        
    def test_send_backlog_err_reliable(self):
        """ Test errors when sending backlog to reliable subscriber. """
        
        class ExcThrowingConn(object):
            reliable_subscriber = True
            def send_frame(self, frame):
                raise RuntimeError("Error sending data.")
        
        dest = '/queue/dest'
        
        f = Frame('SEND', headers={'destination': dest}, body='Empty')
        self.qm.send(f)
        
        conn = ExcThrowingConn()
        try:
            self.qm.subscribe(conn, dest)
            self.fail("Expected error when sending backlog.")
        except RuntimeError:
            pass
        
        # The message will have been requeued at this point, so add a valid
        # subscriber
        
        self.qm.subscribe(self.conn, dest)
        
        print "Frames: %r" % self.conn.frames
        
        assert len(self.conn.frames) == 1, "Expected frame to be delivered"
        assert self.conn.frames[0] == f
        
    def test_send_backlog_err_unreliable(self):
        """ Test errors when sending backlog to reliable subscriber. """
        
        class ExcThrowingConn(object):
            reliable_subscriber = False
            def send_frame(self, frame):
                raise RuntimeError("Error sending data.")
        
        dest = '/queue/dest'
        
        f = Frame('SEND', headers={'destination': dest}, body='123')
        self.qm.send(f)
        
        f2 = Frame('SEND', headers={'destination': dest}, body='12345')
        self.qm.send(f2)
        
        conn = ExcThrowingConn()
        try:
            self.qm.subscribe(conn, dest)
            self.fail("Expected error when sending backlog.")
        except RuntimeError:
            pass
        
        # The message will have been requeued at this point, so add a valid
        # subscriber
        
        print "Queue state: %r" % (self.store._messages,)
        
        self.qm.subscribe(self.conn, dest)
        
        print "Frames: %r" % self.conn.frames
        
        assert len(self.conn.frames) == 2, "Expected frame to be delivered"
        assert self.conn.frames == [f2,f]  
              
    def test_send_reliableFirst(self):
        """
        Test that messages are prioritized to reliable subscribers.
        
        This is actually a test of the underlying scheduler more than it is a test
        of the send message, per se.
        """
        
        dest = '/queue/dest'
        conn1 = MockConnection()
        conn1.reliable_subscriber = True
        
        self.qm.subscribe(conn1, dest)
        
        conn2 = MockConnection()
        conn2.reliable_subscriber = False
        self.qm.subscribe(conn2, dest)
        
        f = Frame('MESSAGE', headers={'destination': dest, 'message-id': uuid.uuid4()}, body='Empty')
        self.qm.send(f)
            
        print conn1.frames
        print conn2.frames
        assert len(conn1.frames) == 1
        assert len(conn2.frames) == 0
    
    def test_ack_basic(self):
        """ Test reliable client (ACK) behavior. """
        
        dest = '/queue/dest'
        conn1 = MockConnection()
        conn1.reliable_subscriber = True
        
        self.qm.subscribe(conn1, dest)
        
        m1 = Frame('MESSAGE', headers={'destination': dest}, body='Message body (1)')
        self.qm.send(m1)
        
        assert conn1.frames[0] == m1
        
        m2 = Frame('MESSAGE', headers={'destination': dest}, body='Message body (2)')
        self.qm.send(m2)
        
        assert len(conn1.frames) == 1, "Expected connection to still only have 1 frame."
        assert conn1.frames[0] == m1
        
        ack = Frame('ACK', headers={'destination': dest, 'message-id': m1.message_id})
        self.qm.ack(conn1, ack)
        
        print conn1.frames
        assert len(conn1.frames) == 2, "Expected 2 frames now, after ACK."
        assert conn1.frames[1] == m2
        
    def test_ack_transaction(self):
        """ Test the reliable client (ACK) behavior with transactions. """
                
        dest = '/queue/dest'
        conn1 = MockConnection()
        conn1.reliable_subscriber = True
        
        self.qm.subscribe(conn1, dest)
        
        m1 = Frame('MESSAGE', headers={'destination': dest, }, body='Message body (1)')
        self.qm.send(m1)
        
        assert conn1.frames[0] == m1
        
        m2 = Frame('MESSAGE', headers={'destination': dest}, body='Message body (2)')
        self.qm.send(m2)
        
        assert len(conn1.frames) == 1, "Expected connection to still only have 1 frame."
        assert conn1.frames[0] == m1
        
        ack = Frame('ACK', headers={'destination': dest, 'transaction': 'abc', 'message-id': m1.message_id})
        self.qm.ack(conn1, ack, transaction='abc')
        
        ack = Frame('ACK', headers={'destination': dest, 'transaction': 'abc', 'message-id': m2.message_id})
        self.qm.ack(conn1, ack, transaction='abc')
        
        assert len(conn1.frames) == 2, "Expected 2 frames now, after ACK."
        assert conn1.frames[1] == m2
        
        self.qm.resend_transaction_frames(conn1, transaction='abc')
        
        assert len(conn1.frames) == 3, "Expected 3 frames after re-transmit."
        assert bool(self.qm._pending[conn1]) == True, "Expected 1 pending (waiting on ACK) frame.""" 
        


"""
Tests for the transport-agnostic engine module.
"""
import unittest

from stompclient.frame import Frame

from coilmq.engine import StompEngine

from coilmq.tests.mock import (MockAuthenticator, MockConnection, MockQueueManager, 
                               MockTopicManager)

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class EngineTest(unittest.TestCase):
    
    def setUp(self):
        self.qm = MockQueueManager()
        self.tm = MockTopicManager()
        self.conn = MockConnection()
        self.auth = MockAuthenticator()
        self.engine = StompEngine(connection=self.conn,
                                  queue_manager=self.qm,
                                  topic_manager=self.tm,
                                  authenticator=None)
    def tearDown(self):
        self.conn.reset()
    
    def _connect(self):
        """ Call the engine connect() method so that we have a valid 'session'. """
        self.engine.connect(Frame('CONNECT'))
    
    def assertErrorFrame(self, frame, msgsub):
        """ Assert that the passed in frame is an error frame and that message contains specified
        string.
        """
        assert frame.command == 'ERROR'
        assert msgsub.lower() in frame.headers['message'].lower()
        
    def test_connect_no_auth(self):
        """ Test the CONNECT command with no auth required. """
        
        assert self.engine.connected == False
        self.engine.process_frame(Frame('CONNECT'))
        assert self.engine.connected == True
        
    def test_connect_auth(self):
        """ Test the CONNECT command when auth is required. """
        self.engine.authenticator = self.auth
        
        assert self.engine.connected == False
        self.engine.process_frame(Frame('CONNECT'))
        self.assertErrorFrame(self.conn.frames[-1], 'Auth')
        assert self.engine.connected == False
        
        self.engine.process_frame(Frame('CONNECT', headers={'login': MockAuthenticator.LOGIN,
                                                                'passcode': MockAuthenticator.PASSCODE}))
        assert self.engine.connected == True
    
    def test_subscribe_noack(self):
        """ Test subscribing to topics and queues w/ no ACK. """
        self._connect()
        self.engine.process_frame(Frame('SUBSCRIBE', headers={'destination': '/queue/bar'}))
        assert self.conn in self.qm.queues['/queue/bar']
        
        self.engine.process_frame(Frame('SUBSCRIBE', headers={'destination': '/foo/bar'}))
        assert self.conn in self.tm.topics['/foo/bar']
    
    def test_send(self):
        """ Test sending to a topic and queue. """
        self._connect()
        
        msg = Frame('SEND', headers={'destination': '/queue/foo'}, body='QUEUEMSG-BODY')
        self.engine.process_frame(msg)        
        assert msg == self.qm.messages[-1]
        
        msg = Frame('SEND', headers={'destination': '/topic/foo'}, body='TOPICMSG-BODY')
        self.engine.process_frame(msg)
        assert msg == self.tm.messages[-1]
        
        msg = Frame('SEND', headers={}, body='TOPICMSG-BODY')
        self.engine.process_frame(msg)
        self.assertErrorFrame(self.conn.frames[-1], 'Missing destination')
    
    def test_subscribe_ack(self):
        """ Test subscribing to a queue with ack=true """
        self._connect()
        self.engine.process_frame(Frame('SUBSCRIBE', headers={'destination': '/queue/bar',
                                                                  'ack': 'client'}))
        assert self.conn.reliable_subscriber == True
        assert self.conn in self.qm.queues['/queue/bar']
        
    def test_unsubscribe(self):
        """ Test the UNSUBSCRIBE command. """
        self._connect()
        self.engine.process_frame(Frame('SUBSCRIBE', headers={'destination': '/queue/bar'}))
        assert self.conn in self.qm.queues['/queue/bar']
        
        print self.conn.frames
        
        self.engine.process_frame(Frame('UNSUBSCRIBE', headers={'destination': '/queue/bar'}))
        assert self.conn not in self.qm.queues['/queue/bar']
        
        print self.conn.frames
        
        self.engine.process_frame(Frame('UNSUBSCRIBE', headers={'destination': '/invalid'}))
        print self.conn.frames[-1]
    
    def test_begin(self):
        """ Test transaction BEGIN. """
        self._connect()
        
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': 'abc'}))
        assert 'abc' in self.engine.transactions
        assert len(self.engine.transactions['abc']) == 0
    
    def test_commit(self):
        """ Test transaction COMMIT. """
        self._connect()
         
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': 'abc'}))
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': '123'}))
        self.engine.process_frame(Frame('SEND', headers={'destination': '/dest', 'transaction': 'abc'}, body='ASDF'))
        self.engine.process_frame(Frame('SEND', headers={'destination': '/dest', 'transaction': 'abc'}, body='ASDF'))
        self.engine.process_frame(Frame('SEND', headers={'destination': '/dest', 'transaction': '123'}, body='ASDF'))
        
        assert len(self.tm.messages) == 0
        
        self.engine.process_frame(Frame('COMMIT', headers={'transaction': 'abc'}))
        
        print self.conn.frames
        
        assert len(self.tm.messages) == 2
        
        assert len(self.engine.transactions) == 1
        
        self.engine.process_frame(Frame('COMMIT', headers={'transaction': '123'}))
        assert len(self.tm.messages) == 3
        
        assert len(self.engine.transactions) == 0
        
    def test_commit_invalid(self):
        """ Test invalid states for transaction COMMIT. """
        self._connect()
        
        # Send a message with invalid transaction 
        f = Frame('SEND', headers={'destination': '/dest', 'transaction': '123'}, body='ASDF')
        self.engine.process_frame(f)
        self.assertErrorFrame(self.conn.frames[-1], 'invalid transaction')
        
        # Attempt to commit invalid transaction
        self.engine.process_frame(Frame('COMMIT', headers={'transaction': 'abc'}))
        
        # Attempt to commit already-committed transaction
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': 'abc'}))
        self.engine.process_frame(Frame('SEND', headers={'destination': '/dest', 'transaction': 'abc'}, body='FOO'))
        self.engine.process_frame(Frame('COMMIT', headers={'transaction': 'abc'}))
        
        self.engine.process_frame(Frame('COMMIT', headers={'transaction': 'abc'}))
        self.assertErrorFrame(self.conn.frames[-1], 'invalid transaction')
    
    def test_abort(self):
        """ Test transaction ABORT. """
        self._connect()
         
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': 'abc'}))
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': '123'}))
        
        f1 = Frame('SEND', headers={'destination': '/dest', 'transaction': 'abc'}, body='ASDF')
        self.engine.process_frame(f1)
        f2 = Frame('SEND', headers={'destination': '/dest', 'transaction': 'abc'}, body='ASDF')
        self.engine.process_frame(f2)
        f3 = Frame('SEND', headers={'destination': '/dest', 'transaction': '123'}, body='ASDF')
        self.engine.process_frame(f3)
        
        assert len(self.tm.messages) == 0
        
        self.engine.process_frame(Frame('ABORT', headers={'transaction': 'abc'}))
        assert len(self.tm.messages) == 0
        
        assert len(self.engine.transactions) == 1
        
    def test_abort_invalid(self):
        """ Test invalid states for transaction ABORT. """
        self._connect()
         
        self.engine.process_frame(Frame('ABORT', headers={'transaction': 'abc'}))
        
        self.assertErrorFrame(self.conn.frames[-1], 'invalid transaction')
        
        self.engine.process_frame(Frame('BEGIN', headers={'transaction': 'abc'}))
        self.engine.process_frame(Frame('ABORT', headers={'transaction': 'abc'}))
        
        self.engine.process_frame(Frame('ABORT', headers={'transaction': 'abc2'}))
        self.assertErrorFrame(self.conn.frames[-1], 'invalid transaction')
        


"""
Classes to be used for mock objects.
"""
from collections import defaultdict

from coilmq import auth

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class MockConnection(object):
    
    def __init__(self):
        self.frames = []
        self.reliable_subscriber = False
        
    def send_frame(self, frame):
        self.frames.append(frame)
    
    def reset(self):
        self.frames = []

class MockAuthenticator(auth.Authenticator):
    LOGIN = 'foo'
    PASSCODE = 'bar'
        
    def authenticate(self, login, passcode):
        return (login == self.LOGIN and passcode == self.PASSCODE)

class MockQueueManager(object):
    
    def __init__(self):
        self.reset()
    
    def reset(self):
        self.queues = defaultdict(set)
        self.acks = []
        self.messages = []
        self.transaction_frames = defaultdict(lambda: defaultdict(list))
        
    def subscribe(self, conn, dest):
        self.queues[dest].add(conn)
    
    def unsubscribe(self, conn, dest):
        if dest in self.queues:
            self.queues[dest].remove(conn)
        
    def send(self, message):
        self.messages.append(message) 
        
    def ack(self, connection, frame, transaction=None):
        
        if transaction:
            self.transaction_frames[connection][transaction].append(frame)
        
        self.acks.append(frame.message_id)
        

    def resend_transaction_frames(self, connection, transaction):
        """ Resend the messages that were ACK'd in specified transaction.
        
        @param connection: The client connection that aborted the transaction.
        @type connection: L{coilmq.server.StompConnection}
        
        @param transaction: The transaction id (which was aborted).
        @type transaction: C{str}
        """
        for frame in self.transaction_frames[connection][transaction]:
            self.send(frame)
    
    def clearTransactionFrames(self, connection, transaction):
        """ Clears out the queued ACK frames for specified transaction. 
        
        @param connection: The client connection that committed the transaction.
        @type connection: L{coilmq.server.StompConnection}
        
        @param transaction: The transaction id (which was committed).
        @type transaction: C{str}
        """
        try:
            del self.transaction_frames[connection][transaction]
        except KeyError:
            pass
        
        
class MockTopicManager(object):
    
    def __init__(self):
        self.reset()
    
    def reset(self):
        self.topics = defaultdict(set)
        self.messages = []
        
    def subscribe(self, conn, dest):
        self.topics[dest].add(conn)
    
    def unsubscribe(self, conn, dest):
        if dest in self.topics:
            self.topics[dest].remove(conn)
        
    def send(self, message):
        self.messages.append(message)


"""
Tests for the scheduler implementation.
"""
import unittest

from coilmq.scheduler import FavorReliableSubscriberScheduler
from coilmq.tests.mock import MockConnection

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class QueueDeliverySchedulerTest(unittest.TestCase):
    """ Tests for various message delivery schedulers. """
     
    def test_favorReliable(self):
        """ Test the favor reliable delivery scheduler. """
        
        sched = FavorReliableSubscriberScheduler()
        
        conn1 = MockConnection()
        conn1.reliable_subscriber = True
        
        conn2 = MockConnection()
        conn2.reliable_subscriber = False
        
        choice = sched.choice((conn1, conn2), None)
        
        assert choice is conn1, "Expected reliable connection to be selected."


"""
Package of available server implementations and shared functionality/interfaces.

CoilMQ is designed for the Python StompServer reference socket server (specifically 
multi-threaded); however, some alternative implementation examples are also provided. 
"""
import abc

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class StompConnection(object):
    """
    An "interface" for server implementation classes to "implement". 
    
    This class serves primarily as a means to document the API that CoilMQ will expect
    the connection object to implement.
    
    @ivar reliable_subscriber: Whether this client will ACK all messages.
    @type reliable_subscriber: C{bool}
    """
    __metaclass__ = abc.ABCMeta
    
    reliable_subscriber = False
    
    @abc.abstractmethod
    def send_frame(self, frame):
        """
        Uses this connection implementation to sends the specified frame to a connected client.
        
        @param frame: The STOMP frame to send.
        @type frame: C{stompclient.frame.Frame}  
        """


"""
Tools to facilitate developing thread-safe components.
"""
__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

def synchronized(func):
    """
    Decorator to lock and unlock a method (Phillip J. Eby).
    
    This function is to be used with object instance methods; the object must
    have a _lock variable (of type C{threading.Lock} or C{threading.RLock}).

    @param func: Method to decorate
    @type func: C{callable}
    """
    def wrapper(self, *__args, **__kw):
        self._lock.acquire()
        try:
            return func(self, *__args, **__kw)
        finally:
            self._lock.release()
    wrapper.__name__ = func.__name__
    wrapper.__dict__ = func.__dict__
    wrapper.__doc__ = func.__doc__
    return wrapper


"""
Simple, lightweight, and easily extensible STOMP message broker.
"""
import os.path
import warnings
import re

try:
    from setuptools import setup, find_packages
except ImportError:
    from distribute_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

version = '0.4.2'

news = os.path.join(os.path.dirname(__file__), 'docs', 'news.txt')
news = open(news).read()
parts = re.split(r'([0-9\.]+)\s*\n\r?-+\n\r?', news)
found_news = ''
for i in range(len(parts)-1):
    if parts[i] == version:
        found_news = parts[i+i]
        break
if not found_news:
    warnings.warn('No news for this version found.')
    
long_description = """
The provided server implementation for CoilMQ uses the Python SocketServer 
libraries; however, CoilMQ is only loosely coupled to this server 
implementation.  It could be used with other socket implementations.

The CoilMQ core classes and bundled storage implementations are built to be
thread-safe.
"""

if found_news:
    title = 'Changes in %s' % version
    long_description += "\n%s\n%s\n" % (title, '-'*len(title))
    long_description += found_news

setup(
    name='CoilMQ',
    version=version,
    description=__doc__,
    long_description=long_description,
    keywords='stomp server broker',
    license='Apache',
    author='Hans Lellelid',
    author_email='hans@xmpl.org',
    url='http://code.google.com/p/coilmq',
    packages=find_packages(exclude=['ez_setup', 'distribute_setup', 'tests', 'tests.*']),
    package_data={'coilmq': ['config/*.cfg*', 'tests/resources/*']},
    zip_safe=False, # We use resource_filename for logging configuration and some unit tests.
    include_package_data=True,
    test_suite='nose.collector',
    tests_require=['nose', 'coverage'],
    install_requires=[
          'distribute',
          'stompclient>=0.2,<0.4',
    ],
    extras_require={
        'daemon': ['python-daemon'],
        'sqlalchemy': ['SQLAlchemy']
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    entry_points="""
    [console_scripts]
    coilmq = coilmq.start:main
    """,
)



"""
Queue storage module that uses thread-safe, in-memory data structures.  
"""
import threading
from collections import defaultdict, deque

from coilmq.store import QueueStore
from coilmq.util.concurrency import synchronized

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class MemoryQueue(QueueStore):
    """
    A QueueStore implementation that stores messages in memory.
    
    This classes uses a C{threading.RLock} to guard access to the memory store.  
    The locks on this class are probably excessive given that the 
    L{coilmq.queue.QueueManager} is already implementing coarse-grained locking 
    on the methods that access this storage backend.  That said, we'll start
    over-protective and refactor later it if proves unecessary. 
    """
    def __init__(self):
        QueueStore.__init__(self)
        self._messages = defaultdict(deque)
    
    @synchronized    
    def enqueue(self, destination, frame):
        self._messages[destination].appendleft(frame)
        
    @synchronized
    def dequeue(self, destination):
        try:
            return self._messages[destination].pop()
        except IndexError:
            return None
    
    @synchronized
    def size(self, destination):
        """
        Size of the queue for specified destination.
        
        @param destination: The queue destination (e.g. /queue/foo)
        @type destination: C{str}
        """
        return len(self._messages[destination])
        
    @synchronized
    def has_frames(self, destination):
        """ Whether this queue has frames for the specified destination. """
        return bool(self._messages[destination])


"""
Module to hold shared SQLAlchemy state.

These objects are set by the L{coilmq.store.sa.init_model} function.
"""
engine = None #: The SA engine
Session = None #: The SA Session (or Session-like callable) 
metadata = None #: The SA C{sqlalchemy.orm.MetaData} instance bound to the engine.



"""
Tests for topic-related functionality.
"""
import unittest

from stompclient.frame import Frame

from coilmq.topic import TopicManager

from coilmq.tests.mock import MockConnection

__authors__ = ['"Hans Lellelid" <hans@xmpl.org>']
__copyright__ = "Copyright 2009 Hans Lellelid"
__license__ = """Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at
 
  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

class TopicManagerTest(unittest.TestCase):
    """ Tests for the L{TopicManager} class. """
    
    def setUp(self):
        self.tm = TopicManager()
        self.conn = MockConnection()
    
    def test_subscribe(self):
        """ Test subscribing a connection to the topic. """
        dest = '/topic/dest'
        
        self.tm.subscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.tm.send(f)
        
        print self.conn.frames
        assert len(self.conn.frames) == 1
        assert self.conn.frames[0] == f
    
    def test_unsubscribe(self):
        """ Test unsubscribing a connection from the queue. """
        dest = '/topic/dest'
        
        self.tm.subscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.tm.send(f)
        
        print self.conn.frames
        assert len(self.conn.frames) == 1
        assert self.conn.frames[0] == f
        
        self.tm.unsubscribe(self.conn, dest)
        f = Frame('MESSAGE', headers={'destination': dest}, body='Empty')
        self.tm.send(f)
        
        assert len(self.conn.frames) == 1
        
    def send_simple(self):
        """ Test a basic send command. """
        dest = '/topic/dest'
        
        f = Frame('SEND', headers={'destination': dest}, body='Empty')
        self.tm.send(f)
        
        # Assert some side-effects
        assert 'message-id' in f.headers
        assert f.command == 'MESSAGE'

